package com.mgyun.shua.engine;

import java.io.DataOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;

import z.hol.shellandroid.Shell;
import z.hol.utils.codec.DigestUtils;
import android.content.Context;
import android.text.TextUtils;

/**
 * 刷机脚本生成器
 * @author holmes
 *
 */
public class ScriptBuilder {
	public static boolean bOldRootPath = true;

	/**
	 * 参数recoveryVersion传入主版本号，如"3" 传入null代表使用谷歌默认的OTA脚本
	 * 
	 * @param shell
	 *            用于保存脚本的shell, 要有root权限
	 */
	public static ScriptBuilder create(Integer recoveryVersion, Shell shell) {
		ScriptBuilder builder = null;
		bOldRootPath = true;
		if (recoveryVersion != null) {
			if (recoveryVersion == 1) {
				bOldRootPath = false;
			} else if (recoveryVersion == 2) {
				builder = new AmendBuilder();
			} else if (recoveryVersion >= 3) {
				builder = new EdifyBuilder();
			}
		}
		if (builder == null) {
			builder = new ScriptBuilder();
		}
		builder.setShell(shell);
		return builder;
	}

	public static String replaceStart(String string, String start,
			String replace) {
		if (string.startsWith(start)) {
			string = replace + string.substring(start.length());
		}
		return string;
	}

	protected String mScript = "";

	private Shell mShell;

	public ScriptBuilder() {

	}

	/**
	 * 设置一个Shell, 要有root权限，主要用于写入脚本
	 * 
	 * @param shell
	 */
	public void setShell(Shell shell) {
		mShell = shell;
	}

	protected void appendLine(String format, Object... args) {
		mScript += String.format(format, args) + "\n";
	}

	/*
	 * public String toString(){ return mScript; }
	 */

	public void backup(String path) {

	}

	/**
	 * OTA只支持两种格式化，分别是/data与/cache
	 */
	public void format(String path) {
		if (path.startsWith("/data") || path.compareToIgnoreCase("data") == 0)
			appendLine("--wipe_data");
		else if (path.startsWith("/cache")
				|| path.compareToIgnoreCase("cache") == 0)
			appendLine("--wipe_cache");
	}

	public void installZip(String path) {
		installZip(path, null);
	}

	public void installZip(String path, String replaceSdcard) {
		if (bOldRootPath) {
			appendLine(FlushUtils.getString(ShuaString.strUpdatePkg),
					AmendBuilder.convertToRootPath(path));
		} else {
			if (TextUtils.isEmpty(replaceSdcard)) {
				appendLine(FlushUtils.getString(ShuaString.strUpdatePkg),
						replaceStart(path, "/mnt/sdcard", "/sdcard"));
			} else {
				String newPath = replaceStart(path, "/mnt/sdcard", "/sdcard");
				newPath = replaceStart(newPath, "/sdcard", replaceSdcard);
				appendLine(FlushUtils.getString(ShuaString.strUpdatePkg),
						newPath);
			}
		}
	}

	public void print(String format, Object... args) {

	}

	public void restore(String path, boolean boot, boolean system,
			boolean data, boolean cache, boolean sdext) {

	}

	public void runProgram(String program, String... args) {

	}

	public boolean saveToCommand(Context context) {
		return saveToCommand(context,
				FlushUtils.getString(ShuaString.strCacheCmd));
		// return saveToCommand(context,
		// Utils.getString(ShuaString.strExtendCmd));
	}

	protected boolean saveToCommand(Context context, String path) {
		Shell cmd = mShell;
		FlushUtils.getChmod(cmd, "/data/local/tmp/command");
		File cmdFile = new File(FlushUtils.getString(ShuaString.strCmdPath));
		if (!cmdFile.exists()) {
			try {
				cmdFile.createNewFile();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		cmdFile = null;
		String cmdPath = FlushUtils.getString(ShuaString.strCmdPath);
		FlushUtils.getChmod(cmd, cmdPath);
		File file = new File(cmdPath);
		FileOutputStream fos = null;
		try {
			fos = new FileOutputStream(file.getAbsolutePath());
		} catch (FileNotFoundException e) {
			e.printStackTrace();
			return false;
		}
		String busyboxPath = FlushUtils.getString(ShuaString.strBusibox);
		// String busyboxPath = (String)Utils.getMapObject("busybox");
		FlushUtils.getChmod(cmd, busyboxPath);
		boolean outputSucessed = false;
		DataOutputStream dos = new DataOutputStream(fos);
		try {
			String str = "ROM Manager and its distribution of ClockworkMod recovery are copyright ClockworkMod,"
					+ " LLC. If you are trying to integrate with ClockworkMod recovery, please build and distribute your own"
					+ " version of ClockworkMod recovery. Unauthorized reverse engineering and usage of ClockworkMod,"
					+ " LLC's ROM Manager and distributed ClockworkMod Recovery are in violation of the DMCA,"
					+ " and will be prosecuted." + mScript;
			String strRet = "echo ";
			/*
			 * MessageDigest md = null; md = MessageDigest.getInstance("MD5");
			 * byte[] ret = md.digest(str.getBytes()); if(ret!=null){ for(byte
			 * b:ret){ strRet+=String.format("%02x", b); } }
			 */
			strRet += DigestUtils.md5Hex(str);

			strRet += FlushUtils.getString(ShuaString.strSaltHash);
			cmd.exec(false, busyboxPath + " mkdir -p /sdcard/clockworkmod;"
					+ strRet);
			// dos.write(mScript.getBytes(""));//"GB2312"
			dos.write(mScript.getBytes("ISO-8859-1"));// "GB2312"
			// dos.write(mScript.getBytes("GBK"));//"GB2312"
			outputSucessed = true;
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			try {
				dos.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		if (!outputSucessed)
			return false;

		String suScript = String.format(
				FlushUtils.getString(ShuaString.strSuScrip), busyboxPath,
				busyboxPath, file.getAbsolutePath(), path);
		cmd.exec(true, suScript, "sync", busyboxPath + " sync");
		return true;
	}
}
