package com.mgyun.shua.general.utils;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import android.content.Context;
import android.content.res.AssetManager;

public class AssetUtils {

	/**
	 * 释放Asset里面的文件
	 * @param context
	 * @param fileName
	 * @param checkFile 如果检测文件，则在文件存在的时候不进行释放
	 * @throws IOException
	 */
	public static void extractAsset(Context context, String fileName, String outname, boolean checkFile) throws IOException{
		if (!checkFile || !isFileExist(context, outname)){
			AssetManager manager = context.getAssets();
			InputStream in = manager.open(fileName);
			OutputStream out = context.openFileOutput(outname, Context.MODE_PRIVATE);
			BufferedOutputStream bout = new BufferedOutputStream(out);
			int iByte = -1;
			while ((iByte = in.read()) != -1){
				bout.write(iByte);
			}
			bout.flush();
			bout.close();
			out.close();
			in.close();
		}
	}
	
	public static boolean isFileExist(Context context, String fileName){
		File fileDir = context.getFilesDir();
		File f = new File(fileDir, fileName);
		boolean exist = f.exists();
		if (exist){
		    String originMd5 = getAssetFileMd5(context, fileName);
		    if (originMd5 != null){
		        FileInputStream in = null;
                try {
                    in = new FileInputStream(f);
                } catch (FileNotFoundException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                }
                if (in != null){
                    String currentMd5 = Du.md5Hex(in);
                    exist = originMd5.equals(currentMd5);
                }
		    }
		}
		f = null;
		fileDir = null;
		return exist;
	}
	
	public static String getAssetFileMd5(Context context, String fileName){
	    AssetManager manager = context.getAssets();
	    InputStream in = null;
	    try {
            in = manager.open(fileName);
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
	    if (in != null){
	        return Du.md5Hex(in);
	    }
	    
	    return null;
	}
}
