package com.mgyun.shua.helper;


import java.io.File;

import z.hol.shellandroid.Shell;
import android.content.Context;
import android.text.TextUtils;

import com.mgyun.shua.engine.DisposeString;
import com.mgyun.shua.engine.FlushUtils;
import com.mgyun.shua.engine.ScriptBuilder;
import com.mgyun.shua.engine.ShuaString;
import com.mgyun.shua.model.FlushData;
import com.mgyun.shua.model.FlushGuider;
import com.yiutil.tools.Logger;
import com.yiutil.tools.MachineUtil;

/**
 * 刷机流程
 * @author holmes
 *
 */
public class FlushHelper {
	
	private Context mContext;
	private FlushGuider mGuider;
	private Shell mShell;
	private ScriptBuilder mBuilder;
	private String mRomPath;
	private int mVersionMajor;
	
	/** 刷机工具路径 */
	private String mFlushToolsPath = FlushData.FILE_FLUSH_SD;
	
	public FlushHelper(Context context, FlushGuider guider, String romPath, Shell shell){
		mContext = context;
		this.mGuider = guider;
		mShell = shell;
		mRomPath = romPath;
	}
	
	/**
	 * 设置刷机工具的路径。默认为 {@link FlushData#FILE_FLUSH_SD}
	 * @param path
	 */
	public void setFlushToolsPath(String path){
		if (!TextUtils.isEmpty(path)){
			mFlushToolsPath = path;
		}else{
			mFlushToolsPath = FlushData.FILE_FLUSH_SD;
		}
	}
	
	/**
	 * 直接执行全部刷机流程。
	 * @param autoReboot
	 */
	public void simpleFlush(boolean autoReboot){
		prepareFlush();
		installRecovery();
		initScript();
		preInstall();
		makeFlushScript();
		saveScript();
		if (autoReboot){
			rebootToFinish();
		}
	}

	/**
	 * 1 刷机准备<br>
	 * 工具文件加入可以执行权限
	 */
	public void prepareFlush(){
		FlushUtils.getChmod(mShell, mContext.getFilesDir().getAbsolutePath());
		File tools = new File(mFlushToolsPath);
		if (tools.exists()){
			FlushUtils.unZipFlush(mContext, tools.getAbsolutePath(), mContext.getFilesDir().getAbsolutePath());
		    FlushUtils.getChmod(mShell, FlushUtils.getString(ShuaString.strReboot));
		    FlushUtils.getChmod(mShell, FlushUtils.getString(ShuaString.strFlashImg));
		    FlushUtils.getChmod(mShell, FlushUtils.getString(ShuaString.strBusibox));
		    FlushUtils.getChmod(mShell, FlushUtils.getString(ShuaString.strRecoveryImg));
		    FlushUtils.getChmod(mShell, FlushData.FILE_RECOVERY_SD);
		}
	}
	
	/**
	 * 2 安装Recovery
	 */
	public void installRecovery(){
		String script = mGuider.getFlushRecovery();
		String dropCache = "echo 3 > /proc/sys/vm/drop_caches;";
		if (!TextUtils.isEmpty(script)){
			script = DisposeString.replaceBusybox(script);
			script = DisposeString.replaceFlashImg(script);
			script = DisposeString.formateDeviceRecovery(script);
		}else{
			script = String.format("%s recovery %s", 
					DisposeString.getFlashImg(),
					FlushData.FILE_RECOVERY
					);
		}
		//TestUtils.LogToFile(script);
		Logger.d("fhelper", script);
		mShell.exec(true, dropCache + script, "sync", DisposeString.getNewSyncString());
	}
	
	/**
	 * 3 准备刷机脚本
	 * @return
	 */
	public boolean initScript(){
//		String version = mGuider.getRecoveryVersion();
//		if (TextUtils.isEmpty(version) && mGuider.recVersion == -1){
//			return false;
//		}
		
		//boolean googleRecovery = mGuider.googleRecovery;
		//mVersionMajor = mGuider.getRecoveryVersion();
//		if (mVersionMajor == -1 )
//			mVersionMajor = Integer.valueOf(version.split("\\.")[0]);
		
//		mBuilder = ScriptBuilder.create(googleRecovery ? null : mVersionMajor);

		mVersionMajor = mGuider.getRecoveryVersion();
		mBuilder = ScriptBuilder.create(mVersionMajor, mShell);
		
		return true;
	}
	
	/**
	 * 是否只能刷一条指令
	 * @return
	 */
	private boolean isOnlyOneFlush(){
		if (mVersionMajor < 2){
			return true;
		}else{
			return false;
		}
	}
	
	/**
	 * 4 刷入Recovery后<br>
	 * 准备PreInstall Script
	 */
	public void preInstall(){
		if (isNeedPreInstall()){
			mBuilder.installZip(FlushData.FILE_PRE_INSTALL, mGuider.getReplaceSdcard());
		}
	}
	
	private boolean isNeedPreInstall(){
		if (mGuider.isNeedPerinsatll()){
			return false;
		}else{
			return true;
		}
	}
	
	/**
	 * 5 编写刷机脚本
	 */
	public void makeFlushScript(){
		installRomScript();
		mBuilder.format(FlushUtils.getString(ShuaString.strCache));
		mBuilder.format(FlushUtils.getString(ShuaString.strData));
		mBuilder.format(FlushUtils.getString(ShuaString.strSdExt));
		if (!isOnlyOneFlush()){
			mBuilder.installZip(FlushData.FILE_ATTACH, mGuider.getReplaceSdcard());
		}
	}
	
	private void installRomScript(){
		String newRomPath = ScriptBuilder.replaceStart(mRomPath, MachineUtil.getSDPath(), "/mnt/sdcard");
		mBuilder.installZip(newRomPath, mGuider.getReplaceSdcard());
	}
	
	/**
	 * 6 保存刷机脚本
	 * @return
	 */
	public boolean saveScript(){
		int saveTimes = 0;
		while (!mBuilder.saveToCommand(mContext)){
			// 保存失败
			saveTimes ++;
			if (saveTimes > 2){
				return false;
			}
		}
		return true;
	}
	
	/**
	 * 7 重启
	 */
	public void rebootToFinish(){
		String rebootScript = mGuider.getRebootRecoverySpecial();
		if (TextUtils.isEmpty(rebootScript)){
			rebootScript = mGuider.getRebootRecovery();
		}
		
		if (!TextUtils.isEmpty(rebootScript)){
			rebootScript = DisposeString.replaceBusybox(rebootScript);
		}else{
			rebootScript = String.format("%s recovery", FlushUtils.getString(ShuaString.strReboot));;
		}
		
		//String originReboot = "reboot recovery";
		//mShell.execute(true, originReboot);
		
		/*try {
			Thread.sleep(3000);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}*/		
		mShell.exec(true, rebootScript);
	}
	
}
