package com.mgyun.shua.model;

import java.io.File;
import java.io.Serializable;
import java.util.Arrays;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import android.os.Environment;
import android.text.TextUtils;

/**
 * 刷机数据.
 * 文件路径中以SD结尾的主要是用于下载的数据处理的路径。
 * 用这个路径在应用用相对安全一些, 像 {@link FlushData#FILE_ATTACH_SD} 。
 * 没有SD结尾的主要是写在刷机脚本中的路径，像 {@link FlushData#FILE_ATTACH}。
 * 绝大多数机子中这两种路径是一样的效果，但有些机子，这两个路径可能不是同一个。
 * 这时代码中一般用带SD卡的
 * 
 * @author holmes
 *
 */
public class FlushData implements Serializable{
	
	private static final long serialVersionUID = -1445327249425776244L;
	
	public static final String KEY_RECOVERY_VERSION= "current_recovery_version";
	
	/** Evironment里面的sdcard目录.api拿目录 */
	public static final String SDCARD_DIR = Environment.getExternalStorageDirectory().getAbsolutePath();
	/** 应用文件目录 */
	public static final String FILE_DIR = "/data/data/com.mgyun.shua/files";

	/** 刷机适配目录, 基于/sdcard */
	public static final String MGYUN_DIR = "/sdcard/mgyun/config";
	/** 刷机适配,主要用于刷机脚本 attach.zip */
	public static final String FILE_ATTACH = MGYUN_DIR + "/attach.zip";
	/** 刷机适配,主要用于刷机脚本 recovery.img */
	public static final String FILE_RECOVERY = MGYUN_DIR + "/recovery.img";
	/** 刷机适配,主要用于刷机脚本 刷机工具 flush.zip */
	public static final String FILE_FLUSH = MGYUN_DIR + "/.flush.zip";
	/** 刷机适配,主要用于刷机脚本 预安装 perinstall.zip */
	public static final String FILE_PRE_INSTALL = MGYUN_DIR + "/preinstall.zip";
	
	/** 刷机适配目录, 基于env拿的sdcard目录 */
	public static final String MGYUN_DIR_SD = SDCARD_DIR + "/mgyun/config";
	/** 刷机适配,用于数据处理下载 attach.zip */
	public static final String FILE_ATTACH_SD = MGYUN_DIR_SD + "/attach.zip";
	/** 刷机适配,用于数据处理下载 recovery.img */
	public static final String FILE_RECOVERY_SD = MGYUN_DIR_SD + "/recovery.img";
	/** 刷机适配,用于数据处理下载 刷机工具 flush.zip */
	public static final String FILE_FLUSH_SD = MGYUN_DIR_SD + "/.flush.zip";
	/** 刷机适配,用于数据处理下载 预安装 perinstall.zip */
	public static final String FILE_PRE_INSTALL_SD = MGYUN_DIR_SD + "/preinstall.zip";
	
	
	
	public String flushUrl = null;
	public String recoveryUrl = null;
	public String attachmentUrl = null;
	private static FlushData sInstance = null;
	
	private FlushData(){
		File fileDir = new File(FILE_RECOVERY);
		if (!fileDir.getParentFile().exists()){
			fileDir.getParentFile().mkdirs();
		}
		fileDir = new File(FILE_FLUSH);
		if (!fileDir.getParentFile().exists()){
			fileDir.getParentFile().mkdirs();
		}
		fileDir = null;
	}
	
	public static FlushData getInstance(){
		if (sInstance == null){
			sInstance = new FlushData();
		}
		return sInstance;
	}
	
	/**
	 * 是否有效.
	 * recovery和flushURL不能为空
	 * @return
	 */
	public boolean isValid(){
		return !(TextUtils.isEmpty(recoveryUrl) || TextUtils.isEmpty(flushUrl));
	}
	
	
	@Override
	public String toString() {
		return "FlushData [flushUrl=" + flushUrl + ", recoveryUrl="
				+ recoveryUrl + ", attachmentUrl=" + attachmentUrl + "]";
	}




	public static final class DeviceModel{
		
		/**
		 * {"name":"",    // 字符串
			"version":"",    // 字符串
			"init":"",       // 字符串
			"key":"",       // 字符串
			"keys":["",""],   // 字符串数组
			"legacy_versions":["",""],     // 字符串数组
			"officially_supported":true,    // 布尔值
			"alternate_recovery":{"clockwork":true,"name":"","url":""},   //对象
			"lunch":"",       // 字符串
			"touch_version":"", //字符串
			"readonly_recovery":true, // 布尔值
			"flash_recovery":"",      //字符串
			"google_recovery":true,   // 布尔值
			"old_root_path":true,     // 布尔值
			"reverse_cmd":true,      // 布尔值
			"test":true,             // 布尔值
			"preinstall":""           // 字符串
			}
		 */
		private boolean mIsValid = true;
		
		public String name;
		public String version;
		public String v4version;
		public String init;
		public String key;
		public String[] keys = null;
		public String[] legacyVersions = null;
		public boolean officiallySupported = false;
		public AlternateRecovery alternateRecovery = null;
		public String lunch = null;
		public String touchVersion = null;
		public boolean readOnlyRecovery = true;
		public String flashRecovery = null;
		public boolean googleRecovery = false;
		public boolean oldRootPath = true;
		public boolean reverseCMD = true;
		public boolean test = true;
		public String preInstall = null;
		public String rebootRecovery = null;
		//public String rebootV2 = null;
		public String recSdcard = null;
		public int recVersion = -1;
		
		/**
		 * 获取一个无效的DeviceModel<br>
		 * 一般用于默认值
		 * @return
		 */
		public static DeviceModel getInvalidDeviceModel(){
			DeviceModel dm = new DeviceModel();
			dm.mIsValid = false;
			return dm;
		}
		
		/**
		 * 获取版本
		 * @param romSDK
		 * @return
		 */
		public String getVersion(String romSDK){
			if (TextUtils.isEmpty(romSDK)){
				return getFinalVersion(false);
			}else{
				if (TextUtils.isEmpty(v4version)){
					return getFinalVersion(false);
				}
				int sdkInt = 0;
				try{
					sdkInt = Integer.valueOf(romSDK);
				} catch (NumberFormatException e) {
					// TODO: handle exception
				}
				
				if (sdkInt >= 14){
					return getFinalVersion(true);
				}else{
					return getFinalVersion(false);
				}
			}
		}
		
		/**
		 * 获取一个最终使用的版本号
		 * @param v4
		 * @return
		 */
		public String getFinalVersion(boolean v4){
			if (recVersion == -1){
				if (v4){
					return v4version;
				}else{
					return version;
				}
			}else{
				return String.valueOf(recVersion);
			}
		}
		
		/**
		 * 是否有Sd卡重定位
		 * @return
		 */
		public boolean hasRecSdcard(){
			return !TextUtils.isEmpty(recSdcard);
		}
		
		/**
		 * 是否有效
		 * @return
		 */
		public boolean isValid(){
			return mIsValid;
		}

		@Override
		public String toString() {
			return "DeviceModel [name=" + name + ", version=" + version
					+ ", init=" + init + ", key=" + key + ", keys="
					+ Arrays.toString(keys) + ", legacyVersions="
					+ Arrays.toString(legacyVersions)
					+ ", officiallySupported=" + officiallySupported
					+ ", alternateRecovery=" + alternateRecovery + ", lunch="
					+ lunch + ", touchVersion=" + touchVersion
					+ ", readOnlyRecovery=" + readOnlyRecovery
					+ ", flashRecovery=" + flashRecovery + ", googleRecovery="
					+ googleRecovery + ", oldRootPath=" + oldRootPath
					+ ", reverseCMD=" + reverseCMD + ", test=" + test
					+ ", preInstall=" + preInstall + ", rebootRecovery="
					+ rebootRecovery + "]";
		}
		
		public static DeviceModel fromJson(String json) throws JSONException{
			JSONObject obj = new JSONObject(json);
			return fromJson(obj);
		}
		
		public static DeviceModel fromJson(JSONObject json) throws JSONException{
			/**
			 * {"name":"",    // 字符串
				"version":"",    // 字符串
				"init":"",       // 字符串
				"key":"",       // 字符串
				"keys":["",""],   // 字符串数组
				"legacy_versions":["",""],     // 字符串数组
				"officially_supported":true,    // 布尔值
				"alternate_recovery":{"clockwork":true,"name":"","url":""},   //对象
				"lunch":"",       // 字符串
				"touch_version":"", //字符串
				"readonly_recovery":true, // 布尔值
				"flash_recovery":"",      //字符串
				"google_recovery":true,   // 布尔值
				"old_root_path":true,     // 布尔值
				"reverse_cmd":true,      // 布尔值
				"test":true,             // 布尔值
				"preinstall":""           // 字符串
				}
			 */
			
			JSONObject obj = json;
			String name = obj.optString("name", null);
			String version = obj.optString("version", null);
			String v4version = obj.optString("v4version", null);
			String init = obj.optString("init", null);
			String key = obj.optString("key", null);
			String[] keys = getStringArrayFromJson(obj, "keys");
			String[] legacyVersions = getStringArrayFromJson(obj, "legacy_versions");
			boolean officiallySupport = obj.optBoolean("officially_supported", false);
			JSONObject recoveryObj = obj.optJSONObject("alternate_recovery");
			AlternateRecovery alternateRecovery = AlternateRecovery.fromJson(recoveryObj);
			String lunch = obj.optString("lunch", null);
			String touchVersion = obj.optString("touch_version", null);
			boolean readOnlyRecovery = obj.optBoolean("readonly_recovery", false);
			String flashRecovery = obj.optString("flash_recovery", null);
			boolean googleRecovery = obj.optBoolean("google_recovery", false);
			boolean oldRootPath = obj.optBoolean("old_root_path", false);
			boolean reverseCMD = obj.optBoolean("reverseCMD", false);
			boolean test = obj.optBoolean("test", false);
			String preInstall = obj.optString("preinstall", null);
			String rebootRecovery = obj.optString("reboot_recovery_V2", null);
			if (TextUtils.isEmpty(rebootRecovery)){
				rebootRecovery = obj.optString("reboot_recovery", null);
			}
			String recSdcard = obj.optString("rec_sdcard", null);
			if (recSdcard != null){
				recSdcard = recSdcard.trim();
			}
			int recVersion = obj.optInt("rec_ver", -1);
			
			DeviceModel md = new DeviceModel();
			md.name = name;
			md.version = version;
			md.v4version = v4version;
			md.init = init;
			md.key = key;
			md.keys = keys;
			md.legacyVersions = legacyVersions;
			md.officiallySupported = officiallySupport;
			md.alternateRecovery = alternateRecovery;
			md.lunch = lunch;
			md.touchVersion = touchVersion;
			md.readOnlyRecovery = readOnlyRecovery;
			md.flashRecovery = flashRecovery;
			md.googleRecovery = googleRecovery;
			md.oldRootPath = oldRootPath;
			md.reverseCMD = reverseCMD;
			md.test = test;
			md.preInstall = preInstall;
			md.rebootRecovery = rebootRecovery;
			md.recSdcard = recSdcard;
			md.recVersion = recVersion;
			return md;
		}
		
		private static String[] getStringArrayFromJson(JSONObject obj, String arrayKey) throws JSONException{
			String[] strArray = null;
			if (!obj.has(arrayKey) || obj.get(arrayKey) == JSONObject.NULL){
				return null;
			}
			JSONArray a = obj.getJSONArray(arrayKey);
			int count = a.length();
			strArray = new String[count];
			for (int i = 0; i < count; i ++){
				strArray[i] = a.getString(i);
			}
			return strArray;
		}

		public static final class AlternateRecovery{
			public boolean clockwork;
			public String name;
			public String url;
			@Override
			public String toString() {
				return "AlternateRecovery [clockwork=" + clockwork + ", name="
						+ name + ", url=" + url + "]";
			}
			
			public static AlternateRecovery fromJson(JSONObject obj){
				if (obj == null){
					return null;
				}
				AlternateRecovery alternateRecovery = new AlternateRecovery();
				alternateRecovery.clockwork = obj.optBoolean("clockwork", false);
				alternateRecovery.name = obj.optString("name", null);
				alternateRecovery.url = obj.optString("url", null);
				return alternateRecovery;
			}
		}
	}
}
