package com.mgyun.shua.protector;

/**
 * 可取消的任务
 * @author holmes
 *
 */
public interface CancelableTask {
	
	/**
	 * 可取消任务的运行状态
	 * @author holmes
	 *
	 */
	public static enum Status{
		NORMAL, RUNNING, FINISHED, CANCELING, CANCELED
	}
	
	/**
	 * 取消回调
	 * @author holmes
	 *
	 */
	public static interface OnCancelListener{
		
		/**
		 * 取消回调
		 * @param task
		 */
		public void onCancel(CancelableTask task);
	}
	
	/**
	 * 取消
	 */
	public void cancel();
	
	/**
	 * 是否已经取消,
	 * 和运行状态 {@link Status} 无联系，
	 * 只表示取消标识位的值
	 * @return
	 */
	public boolean isCanceled();
	
	/**
	 * 获取当前运行状态
	 * @return
	 */
	public Status getCurrentStatus();
	
	/**
	 * 设置取消回调
	 * @param listener
	 */
	public void setOnCancelListener(OnCancelListener listener);
}
