package com.mgyun.shua.sdk;

import java.io.File;

import android.content.Context;
import android.text.TextUtils;

import com.mgyun.shua.general.utils.Decoder;
import com.mgyun.shua.su.sdk.core.SDKDaddy;
import com.mgyun.shua.su.sdk.core.SuBinaryHelper;

/**
 * 初始化
 * @author holmes
 *
 */
public final class MgyRoot {
	public static final String TAG = "mgyunSdk";

    public static final int VERSION = 10011702;	// 1.1.7
    public static final String DEFUALT_SUPERUSER_ASSET_NAME = "mgyunsuser";
    public static final String DEFUALT_SUPERUSER_NAME = "Superuser";
    
    /** 过期时间 */
    public static final long OT = 0l;	// last 1456156800000l
    
    private static MgyRoot sInstance = null;
    
    public static MgyRoot getInstance(){
        if (sInstance == null){
            sInstance = new MgyRoot();
        }
        return sInstance;
    }
    
    private boolean mIsInited = false;
    private File mLibDir = null;
    private Context mContext;
    
    private boolean mPushSelfToSystem = false;
    private String mSuperuserName = DEFUALT_SUPERUSER_NAME;
    
    private String mSuperuserApkPath = null;
    private boolean mUserMgyunSuApk = false;
    
    /** 要释放的su 在asset里面的名称 */
    private String mSuAssetName = null;

    /** ROOT方案 */
    private String[] mRootTaskPath;
    /** cflag 的配置路径 */
    private String mCflagFilePath;
    
    /**  */
    private String mRootKey = null;
    
    /** 使用安全方案 */
    private boolean mIsSafeMode = false;

    /** 用于获取在线token的appid */
    private String mAppId;

    /**
     * 初始化
     * @param context
     * @return
     */
    public boolean init(Context context){
        mContext = context.getApplicationContext();
        if (!SDKDaddy.isDecoded()) {
        	Decoder.decodeClass(SDKDaddy.class);
        }
        mLibDir = getlibDir(mContext);
//        try {
//            CoreUtils.ensureJni(mContext);
//        } catch (RootException e) {
//            // It is Auto-generated catch block
//            e.printStackTrace();
//            return false;
//        }
        
        // 设置APK
        
        mIsInited = true;
        return mIsInited;
    }
    
    /**
     * 是否使用mgyun的Superuser APk
     * @return
     */
    public boolean isUserMgyunSuApk(){
        return mUserMgyunSuApk;
    }
    
    /**
     * 自动设置mgyun默认的Superuser apk路径名称.
     * 会自动释放
     */
    public void setSuperuserApkPathForMgyun(Context context){
        File fileDir = context.getFilesDir();
        setSuperuserApkPath(new File(fileDir, DEFUALT_SUPERUSER_ASSET_NAME).getAbsolutePath());
        mUserMgyunSuApk = true;
    }
    
    /**
     * 设置Superuser Apk的路径, 如果 {@link #isPushSelfApkToSystem()}, 则会把
     * Apk 推入到 system/app里面.<br>
     * Root时, Apk文件要存在才有效
     */
    public void setSuperuserApkPath(String path){
        mSuperuserApkPath = path;
        if (TextUtils.isEmpty(path)){
            throw new IllegalArgumentException("path should not be blank (null or \"\")");
        }
        mUserMgyunSuApk = false;
    }
    
    /**
     * 获取Superuser的Apk路径
     * @return
     */
    public String getSuperuserApkPath(){
        return mSuperuserApkPath;
    }
    
    /**
     * 是否已经初始化好
     * @return
     */
    public boolean isInited(){
        return mIsInited;
    }
    
    /**
     * 自动设置mgyun默认的su名称
     */
    public void setSuAssetNameForMgyun(){
        String su = SuBinaryHelper.ensureSuName();
        setSuAssetName(su);
    }
    
    /**
     * 设置要释放的su在asset里面的名称,
     * root时会将这个su,释放到apk的file目录下, 名为"su"
     * @param su
     */
    public void setSuAssetName(String su){
        mSuAssetName = su;
    }
    
    /**
     * 获取要释放的su在asset里面的名称
     * @param su
     */
    public String getSuAssetName(){
        return mSuAssetName;
    }
    
    /**
     * 获取lib目录
     * @return
     */
    public File getLibDir(){
        return mLibDir;
    }
    
    /**
     * 获取lib目录路径
     * @return
     */
    public String getLibPath(){
        if (mLibDir != null){
            return mLibDir.getAbsolutePath();
        }
        return null;
    }
    
    /**
     * 获取Superuser的名称, 当 {@link #isPushSelfApkToSystem()} 有效
     * @return
     */
    public String getSuperuserName(){
        return mSuperuserName;
    }
    
    /**
     * 设置Superuser的名称, 当 {@link #isPushSelfApkToSystem()} 有效
     * @return
     */
    public void setSuperuserName(String name){
        if (!TextUtils.isEmpty(name)){
            mSuperuserName = name;
        }else{
            throw new IllegalArgumentException("name should not be blank (null or \"\")");
        }
    }
    
    /**
     * 设置在Root成功后是否将自己推送到System/app目录下
     * @param push
     */
    public void setPushSelfApkToSystem(boolean push){
        mPushSelfToSystem = push;
    }
    
    /**
     * Root成功后是否将自己推送到System/app目录下
     * @param push
     */
    public boolean isPushSelfApkToSystem(){
        return mPushSelfToSystem;
    }
    
    /**
     * 获取ROOT方案
     * @return
     */
    public String[] getRootTasks(){
    	return mRootTaskPath;
    }
    
    /**
     * 设置ROOT方案
     * @param rootTasks
     * @return
     */
    public void setRootTasks(String[] rootTasks){
    	mRootTaskPath = rootTasks;
    }
    
    /**
     * 获取ROOT方案数量
     */
    public int getRootTaskCount(){
    	final int len = mRootTaskPath != null ? mRootTaskPath.length : 0;
    	return len;
    }
    
    /**
     * 获取索引一个ROOT方案
     * @param index
     * @return
     */
    public String getRootTaskPath(int index){
    	final int len = mRootTaskPath != null ? mRootTaskPath.length : 0;
    	if (index >= 0 && index < len){
    		return mRootTaskPath[index];
    	}
    	return null;
    }
    
    /**
     * 设置ROOT key
     * @param key
     */
    public void setRootKey(String key){
    	mRootKey = key;
    }
    
    /**
     * 获取ROOT KEY
     * @return
     */
    public String getRootKey(){
    	return mRootKey;
    }
    
    public boolean isSafeMode(){
    	return mIsSafeMode;
    }
    
    public void setSafeMode(boolean safe){
    	mIsSafeMode = safe;
    }
    
    /**
     * 设置Shell的Cflag路径
     * @param path
     */
    public void setCFlagPath(String path){
    	mCflagFilePath = path;
    }
    
    public String getCFlagPath(){
    	return mCflagFilePath;
    }
    
    /**
     * 是否使用自定义的cflag路径
     * @return
     */
    public boolean isUserCustomCFlag(){
    	return !TextUtils.isEmpty(mCflagFilePath);
    }

    /**
     * 获取设置的appid
     * @return
     */
    public String getAppId() {
    	return mAppId;
        // return "mgyun117";
    }

    /**
     * 设置appid
     * @param appId
     */
    public void setAppId(String appId) {
        mAppId = appId;
    }

    /**
     * 获取Lib库的目录
     * @param context
     * @return
     */
    public static File getlibDir(Context context){
        File dir = context.getDir("lib", Context.MODE_WORLD_READABLE);
        return dir;
    }
}
