package com.mgyun.shua.su.sdk.core;

import java.io.File;
import java.io.IOException;

import z.hol.shellandroid.ShellAndroid;
import z.hol.shellandroid.exception.ShellExecuteException;
import z.hol.shellandroid.utils.ShellUtils;
import android.content.Context;
import android.text.TextUtils;

import com.mgyun.shua.core.Eva;
import com.mgyun.shua.core.RootException;
import com.mgyun.shua.general.utils.AssetUtils;
import com.mgyun.shua.general.utils.IntervalData;
import com.mgyun.shua.sdk.MgyRoot;
import com.mgyun.shua.sdk.RootCracker;

/**
 * 大杀器Root
 *
 * @author holmes
 */
public class WOMDRootCracker implements RootCracker {


	
    private ShellAndroid mShell;
    private Context mContext;
    private String mApkPath;
    private String mAssetSuName = null;
    /**
     * 是否需要释放su
     */
    private final boolean mExtractSu;

    private String mRootKey;
    private String mAppId;

    private boolean mIsSafeMode = false;

    public WOMDRootCracker(Context context) {
        mContext = context.getApplicationContext();
        String cflag = MgyRoot.getInstance().getCFlagPath();
        boolean customCflag = MgyRoot.getInstance().isUserCustomCFlag();

        mShell = new ShellAndroid(null);
        String flagFile;
        if (customCflag) {
            File cFlagFile = new File(cflag);
            flagFile = mShell.initFlag(mContext, cFlagFile);
        } else {
            flagFile = mShell.initFlagMinimum(mContext);
        }

        mShell.setFlagFile(flagFile);
        mShell.printOutput();
        mShell.setCheckSu(false);

        mRootKey = MgyRoot.getInstance().getRootKey();
        mAppId = MgyRoot.getInstance().getAppId();
        mApkPath = MgyRoot.getInstance().getSuperuserApkPath();
        if (TextUtils.isEmpty(mApkPath)) {
            mApkPath = mContext.getPackageResourcePath();
        }

        mAssetSuName = MgyRoot.getInstance().getSuAssetName();
        mExtractSu = !TextUtils.isEmpty(mAssetSuName);

        mIsSafeMode = MgyRoot.getInstance().isSafeMode();
    }

    @Override
    public int root() throws RootException {
        // TODO Auto-generated method stub
        if (MgyRoot.OT > 1000l && System.currentTimeMillis() > MgyRoot.OT) {
            throw new RootException("");
        }

        if (!MgyRoot.getInstance().isInited()) {
            throw new RootException("mgyun root never inited");
        }

        if (MgyRoot.getInstance().isUserCustomCFlag()) {
            // 使用自定义cflag, 但是文件不存在
            String cflag = MgyRoot.getInstance().getCFlagPath();
            File cFlagFile = new File(cflag);
            if (!cFlagFile.exists()) {
                throw new RootException("cflag file not exist, can't create shell. " + cflag);
            }
        }

        if (TextUtils.isEmpty(mRootKey)) {
        	
            throw new RootException("Root key is empty");
        }

        boolean isKeyAvild = Eva.initSolid(mRootKey);
        if (!isKeyAvild) {
            throw new RootException("Root key is invalid. key -- " + mRootKey);
        }

        int count = getTestCount();
        //System.out.println("count is: " + count);


        File dir = mContext.getFilesDir();
        RootTestPlan[] planClasses;
        if (count > 0) {
            planClasses = new RootTestPlan[count];
            for (int i = 0; i < count; i++) {
                if (mIsSafeMode) {
                    // 安全模式下 屏蔽最后一条方案
                    if (i == count - 1) {
                        continue;
                    }
                }

                planClasses[i] = new DynamicRoot(dir, String.valueOf(i));
            }
        } else {
            planClasses = new RootTestPlan[]{
                    new Root1(dir),
                    new Root2(dir),
                    new Root3(dir),
                    new Root4(dir)
            };
        }

        int result = -1;
        boolean success = false;
        for (int i = 0; i < planClasses.length; i++) {
            RootTestPlan root = planClasses[i];
            if (root != null) {
                try {
                    root.root();
                    result = putSuFile(mShell, dir);
                } catch (ShellExecuteException e) {
                    // This is Auto-generated catch block
                    e.printStackTrace();
                }
                success = isRootSuccess(result);
                root.finish();
                if (success) {
                    break;
                }
            }
        }
        return result;
    }

    /**
     * 获取Count
     *
     * @return
     * @throws RootException
     */
    private int getTestCount() throws RootException {
        //test 授权码 KEY
        final boolean isDebug = ShellAndroid.DEBUG;
        if (mShell != null) {
            ShellAndroid.DEBUG = false;
            String key = null;
            try {
                key = Eva.solid();
            } catch (Exception e) {
                throw new RootException("Root security error", e);
            }

            String rootTest = MgyRoot.getInstance().getRootTaskPath(0);
            mShell.exec(false, "chmod 777 " + rootTest);
            String cmd = rootTest + " " + mRootKey + " " + key;
            try {
                Thread.sleep(1000l);
            } catch (InterruptedException e1) {
                // This is Auto-generated catch block
                e1.printStackTrace();
            }
            mShell.exec(false, cmd);
            String result = mShell.getLastResult();
            int count = 0;
            if (!TextUtils.isEmpty(result)) {
                System.out.println("c: " + result);
                result = result.trim();
                int index = result.indexOf("c=");
                if (index > -1) {
                    String countStr = result.substring(index + 2);
                    try {
                        count = Integer.parseInt(countStr);
                    } catch (NumberFormatException e) {
                        // This is Auto-generated catch block
                        //e.printStackTrace();
                    }
                }
            } else {
                System.out.println("c: <is null>");
            }
            ShellAndroid.DEBUG = isDebug;
            return count;
        }
        return 0;
    }

    @SuppressWarnings("unused")
    private int checkRoot(ShellAndroid shell) {
        shell.checkRoot();
        if (shell.hasRoot()) {
            return 1;
        }
        return 0;
    }

    /**
     * Root成功后, 推入Su文件
     *
     * @param shell   Shell
     * @param fileDir files文件的目录
     * @return
     */
    private int putSuFile(final ShellAndroid shell, File fileDir) {
        Thread rootThread = new Thread() {
            @Override
            public void run() {
                // This is Auto-generated method stub
                try {
                    shell.checkRoot();
                } catch (ShellExecuteException e) {
                    // This is Auto-generated catch block
                    e.printStackTrace();
                }
            }
        };
        rootThread.start();
        try {
            rootThread.join(30000);
        } catch (InterruptedException e) {
            // This is Auto-generated catch block
            e.printStackTrace();
        }

        if (shell.hasRoot()) {
            int id = shell.checkId();
            if (id == 0) {
                if (mExtractSu) {
                    String suPath = fileDir.getAbsolutePath() + "/su";
                    shell.exec(false, "mount -o remount, rw /system");
                    shell.exec(false, suPath + " -r /system");
                    //mShell.exec(false, "cat " + dir.getAbsolutePath() + "/su" + " > /system/bin/su");
                    copyByCat(shell, suPath, "/system/bin/su");
                    shell.exec(false, "chmod 6755 /system/bin/su");
                }

                //mShell.exec(false, "cat " + mApkPath + " > /system/app/Superuser.apk");
                MgyRoot mgyRoot = MgyRoot.getInstance();
                if (mgyRoot.isUserMgyunSuApk()) {
                    // 释放 apk
                    try {
                        AssetUtils.extractAsset(mContext, MgyRoot.DEFUALT_SUPERUSER_ASSET_NAME, MgyRoot.DEFUALT_SUPERUSER_ASSET_NAME, true);
                    } catch (IOException e) {
                        // It is Auto-generated catch block
                        e.printStackTrace();
                    }
                }
                if (mgyRoot.isPushSelfApkToSystem()) {
                    shell.exec(false, "sync");
                    String superuserPath = "/system/app/" + mgyRoot.getSuperuserName() + ".apk";
                    copyByCat(shell, mApkPath, superuserPath);
                    shell.exec(false, "chmod 644 " + superuserPath);
                    shell.exec(false, "sync");
                    shell.exec(false, "pm install -r " + superuserPath);
                }
            }
            //Toast.makeText(mContext, "Root成功", Toast.LENGTH_SHORT).show();;
        } else {
            //Toast.makeText(mContext, "Root失败", Toast.LENGTH_SHORT).show();;
            return 0;
        }
        return 1;
    }

    /**
     * 复制通用Cat命令
     *
     * @param shell
     * @param from
     * @param to
     */
    private static void copyByCat(ShellAndroid shell, String from, String to) {
        StringBuilder sb = new StringBuilder(128);
        sb.append("cat");
        sb.append(" ");
        sb.append(from);
        sb.append(" > ");
        sb.append(to);
        shell.exec(false, sb.toString());
    }

    @Override
    public boolean isRootSuccess(int rootResultCode) {
        // TODO Auto-generated method stub
        return rootResultCode == 1;
    }

    @Override
    public ShellAndroid getShell() {
        // It is Auto-generated method stub
        return mShell;
    }

    @Override
    public void finish() {
        // TODO Auto-generated method stub
        mShell.close();
    }

    /**
     * 破解测试方案(test方案)
     *
     * @author holmes
     */
    private class RootTestPlan implements RootCracker {
        @SuppressWarnings("unused")
        private File mFileDir;
        private final String mCrackerExecutePath;
        private final String mIndex;

        public RootTestPlan(File fileDir, String crackerExecutePath, String index) {
            // TODO Auto-generated constructor stub
            mFileDir = fileDir;
            mCrackerExecutePath = crackerExecutePath;
            mIndex = index;
        }

        @Override
        public int root() throws RootException {
            // TODO Auto-generated method stub
            String rootExecutePath = mCrackerExecutePath;
            // 不在自动释放root方案
            if (TextUtils.isEmpty(rootExecutePath)) {
                throw new RootException("Root task is null");
            }

            if (mExtractSu) {
                try {
                    AssetUtils.extractAsset(mContext, mAssetSuName, SuBinaryHelper.DEFAULT_SU_NAME, true);
                } catch (IOException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                }
            }

            String cmdFile = rootExecutePath;
            File rootPlanFile = new File(cmdFile);
            if (!rootPlanFile.exists()) {
                throw new RootException("Root task no found. " + cmdFile);
            }

            ShellUtils.setChmod(mShell, cmdFile, "777");
            if (mExtractSu) {
                String suFile = mContext.getFileStreamPath(SuBinaryHelper.DEFAULT_SU_NAME).getAbsolutePath();
                ShellUtils.setChmod(mShell, suFile, "777");
            }
            String key = null;
            try {
                key = Eva.solid();
            } catch (Exception e) {
                throw new RootException("Root security error", e);
            }

            // test 授权码 KEY 序号 /system/bin/sh
            StringBuilder cmdBuilder = new StringBuilder(128);
            cmdBuilder.append(cmdFile);
            cmdBuilder.append(" ");
            cmdBuilder.append(mRootKey);
            cmdBuilder.append(" ");
            cmdBuilder.append(key);
            cmdBuilder.append(" ");
            cmdBuilder.append(mIndex);
            cmdBuilder.append(" ");
            cmdBuilder.append("/system/bin/sh");

            //String rootCmd = cmdFile + " " + key + " " + " /system/bin/sh";
            String rootCmd = cmdBuilder.toString();
            // ROOT执行任务
            // 主要是执行一些可能被永久阻塞的命令
            RootShellThread rootThread = new RootShellThread(mShell, false, rootCmd) {
                @Override
                public void run() {
                    // This is Auto-generated method stub
                    super.run();
                    try {
                        mShell.checkRoot();
                    } catch (ShellExecuteException e) {
                        // This is Auto-generated catch block
                        e.printStackTrace();
                    }
                }
            };
            rootThread.start();
            try {
                // 如果一分半还没有ROOT完 , 则出错
                rootThread.join(90000);
            } catch (InterruptedException e) {
                // This is Auto-generated catch block
                e.printStackTrace();
            }

            return mShell.hasRoot() ? 1 : 0;
        }

        @Override
        public boolean isRootSuccess(int rootResultCode) {
            // TODO Auto-generated method stub
            return rootResultCode == 1;
        }

        @Override
        public void finish() {
            // TODO Auto-generated method stub

        }

        @Override
        public ShellAndroid getShell() {
            // It is Auto-generated method stub
            return mShell;
        }
    }

    /**
     * 方案1
     *
     * @author holmes
     */
    private class Root1 extends RootTestPlan {


        public Root1(File fileDir) {
            // TODO Auto-generated constructor stub
            super(fileDir, MgyRoot.getInstance().getRootTaskPath(0), "0");
        }

    }

    /**
     * 方案2
     *
     * @author holmes
     */
    private class Root2 extends RootTestPlan {

        public Root2(File fileDir) {
            // TODO Auto-generated constructor stub
            super(fileDir, MgyRoot.getInstance().getRootTaskPath(0), "1");
        }

    }

    /**
     * 方案3
     *
     * @author holmes
     */
    private class Root3 extends RootTestPlan {

        public Root3(File fileDir) {
            // TODO Auto-generated constructor stub
            super(fileDir, MgyRoot.getInstance().getRootTaskPath(0), "2");
        }

    }

    /**
     * 方案4
     *
     * @author holmes
     */
    private class Root4 extends RootTestPlan {

        public Root4(File fileDir) {
            // TODO Auto-generated constructor stub
            super(fileDir, MgyRoot.getInstance().getRootTaskPath(0), "3");
        }

    }

    /**
     * 动态ROOT方案
     *
     * @author holmes
     */
    private class DynamicRoot extends RootTestPlan {

        /**
         * 动态ROOT方案, 方案 index
         *
         * @param fileDir
         * @param index
         */
        public DynamicRoot(File fileDir, String index) {
            super(fileDir, MgyRoot.getInstance().getRootTaskPath(0), index);
            // This is Auto-generated constructor stub
        }

    }
}
