package com.yiutil.tools;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;

import android.content.ContentResolver;
import android.database.Cursor;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Matrix;
import android.graphics.Rect;
import android.graphics.BitmapFactory.Options;
import android.media.ExifInterface;
import android.net.Uri;
import android.provider.MediaStore;
import android.provider.MediaStore.Images;

/** @类名：BitmapUtil
 *  @创建：
 *  @创建日期：
 *  @说明：
 ***************************************************/
public final class BitmapUtil{
	public static final int UNCONSTRAINED = -1;  
	public static Bitmap decodeFile(String filepath, int size, boolean square){
		return decodeFile(new File(filepath),size,square);
	}
	
	public static Bitmap decodeFile(File file, int size, boolean square){
        BitmapFactory.Options bitmapOptions = new BitmapFactory.Options();
        bitmapOptions.inJustDecodeBounds = true;
        if(size>0){
            int width_tmp=bitmapOptions.outWidth, height_tmp=bitmapOptions.outHeight;
            int scale = 1;
            while(true){
                if(width_tmp/2<size || height_tmp/2<size)
                    break;
                width_tmp/=2;
                height_tmp/=2;
                scale++;
            }
			//decode with inSampleSize
            bitmapOptions = new BitmapFactory.Options();
            bitmapOptions.inSampleSize=scale;
			bitmapOptions.inScaled = true;
			if(square){
            	return cropToSquare(BitmapFactory.decodeFile(file.getAbsolutePath(), bitmapOptions));
            }
            return BitmapFactory.decodeFile(file.getAbsolutePath(), bitmapOptions);
        }
        return null;
	}
	
	public static Bitmap cropToSquare(Bitmap bitmap){
		if(bitmap!=null){
		    if(bitmap.getWidth()>bitmap.getHeight()){
				return Bitmap.createBitmap(bitmap,
						((bitmap.getWidth()-bitmap.getHeight())/2), 0, bitmap.getHeight(), bitmap.getHeight());
			}else if(bitmap.getWidth()<bitmap.getHeight()){
				return Bitmap.createBitmap(bitmap, 0,
						((bitmap.getHeight()-bitmap.getWidth())/2), bitmap.getWidth(), bitmap.getWidth());
			}
	    }
		return bitmap;
	}
	
	public static Bitmap getThumbnail(ContentResolver contentResolver, long id){
		Cursor cursor = contentResolver.query(MediaStore.Images.Media.EXTERNAL_CONTENT_URI,
	             	new String[]{MediaStore.Images.Media.DATA}, // Which columns to return 
	             	MediaStore.Images.Media._ID+ "=?",       // Which rows to return 
	             	new String[]{String.valueOf(id)},       // Selection arguments
	             	null);// order
		
		if(cursor!=null && cursor.getCount()>0){
			cursor.moveToFirst();
			String filepath = cursor.getString(0);
			cursor.close();
			int rotation = 0;

			try{
				ExifInterface exifInterface = new ExifInterface(filepath);
				int exifRotation = exifInterface.getAttributeInt(
						ExifInterface.TAG_ORIENTATION,ExifInterface.ORIENTATION_UNDEFINED);
				if(exifRotation!=ExifInterface.ORIENTATION_UNDEFINED){
					switch(exifRotation){
						case ExifInterface.ORIENTATION_ROTATE_180 :
							rotation = 180;
							break;
						case ExifInterface.ORIENTATION_ROTATE_270 :
							rotation = 270;
							break;
						case ExifInterface.ORIENTATION_ROTATE_90 :
							rotation = 90;
							break;
					}
				}
			}catch(java.io.IOException e){}
			
			Bitmap bitmap = MediaStore.Images.Thumbnails.getThumbnail(
					contentResolver, id, MediaStore.Images.Thumbnails.MINI_KIND, null);
			
			if(rotation!=0){
				Matrix matrix = new Matrix();
				matrix.setRotate(rotation);
				bitmap = Bitmap.createBitmap(bitmap, 0, 0, bitmap.getWidth(), bitmap.getHeight(), matrix, true);
			}
			
			return bitmap;
		}else{
			return null;
		}
	}
	
	/**
	 * 根据视频在手机中的地址路径取得指定的视频缩略图
	 * @param cr  
	 * @param fileName  本地视频地址
	 * @return 返回bitmap类型数据
	 */
    public static Bitmap getVideoThumbnail(ContentResolver cr, Uri uri){  
        Bitmap bitmap = null;  
        BitmapFactory.Options options = new BitmapFactory.Options();  
        options.inDither = false;  
        options.inPreferredConfig = Bitmap.Config.ARGB_8888;  
        Cursor cursor = cr.query(uri,new String[] { MediaStore.Video.Media._ID }, null, null, null);   
        if (cursor == null || cursor.getCount() == 0) {  
            return null;  
        }  
        cursor.moveToFirst();  
        //image id in image table.s  
        String videoId = cursor.getString(cursor.getColumnIndex(MediaStore.Video.Media._ID)); 
  
        if (videoId == null) {  
        	return null;  
        }  
        cursor.close();  
        long videoIdLong = Long.parseLong(videoId);  
        bitmap = MediaStore.Video.Thumbnails.getThumbnail(
        		cr, videoIdLong,Images.Thumbnails.MICRO_KIND, options);  
        return bitmap;  
     } 
    
    /** 
     * 加载大图片：解决android加载大图片时报OOM异常 
     * 解决原理：先设置缩放选项，再读取缩放的图片数据到内存，规避了内存引起的OOM 
     * 获得设置信息 
     */  
    public static Options getOptions(String path){  
	     Options options = new Options();  
	     options.inJustDecodeBounds = true;//只描边，不读取数据  
	     BitmapFactory.decodeFile(path, options);  
	     return options;  
    } 
    
    /** 
     * 获得图像 
     * @param path 
     * @param options 
     * @return 
     * @throws FileNotFoundException 
     */  
    public static Bitmap getBitmapByPath(String path, Options options,
    		         int screenWidth,int screenHeight)throws FileNotFoundException{  
         File file = new File(path);  
         if(!file.exists()){  
             throw new FileNotFoundException();  
         }  
         FileInputStream in = null;  
         in = new FileInputStream(file);  
	     if(options != null){  
	    	 Rect r = getScreenRegion(screenWidth,screenHeight);  
	    	 int w = r.width();  
	    	 int h = r.height();  
	    	 int maxSize = w > h ? w : h;  
	    	 int inSimpleSize = computeSampleSize(options, maxSize, w * h);  
	    	 options.inSampleSize = inSimpleSize; //设置缩放比例  
	    	 options.inJustDecodeBounds = false;  
	     }  
	     Bitmap b = BitmapFactory.decodeStream(in, null, options);  
	     try{  
	    	 in.close();  
	     } catch (IOException e){  
	    	 e.printStackTrace();  
	     }  
	     return b;  
    }  
    
    public static Rect getScreenRegion(int width , int height){  
    	 return new Rect(0,0,width,height);  
    }  
    
    /** 
     * 获取需要进行缩放的比例，即options.inSampleSize 
     * @param options 
     * @param minSideLength 
     * @param maxNumOfPixels 
     * @return 
     */  
    public static int computeSampleSize(BitmapFactory.Options options,  
               int minSideLength, int maxNumOfPixels){  
    	int initialSize = computeInitialSampleSize(options, minSideLength,maxNumOfPixels);  
    	int roundedSize;  
    	if (initialSize <= 8){  
    		roundedSize = 1;  
    		while (roundedSize < initialSize){  
    			roundedSize <<= 1;  
    		}  
    	}else{  
    		roundedSize = (initialSize + 7) / 8 * 8;  
    	}  
    	return roundedSize;  
    }  
     
    private static int computeInitialSampleSize(BitmapFactory.Options options,  
               int minSideLength, int maxNumOfPixels){  
        double w = options.outWidth;  
        double h = options.outHeight;  
 
        int lowerBound = (maxNumOfPixels == UNCONSTRAINED) ? 1:(int) Math.ceil(Math.sqrt(w * h / maxNumOfPixels));  
        int upperBound = (minSideLength == UNCONSTRAINED) ? 128 :  
               (int) Math.min(Math.floor(w / minSideLength),Math.floor(h / minSideLength));  
 
        if (upperBound < lowerBound){  
            return lowerBound;  
        }  
 
        if ((maxNumOfPixels == UNCONSTRAINED)&&(minSideLength == UNCONSTRAINED)) {  
            return 1;  
        }else if (minSideLength == UNCONSTRAINED) {  
            return lowerBound;  
        } else{  
            return upperBound;  
        }  
    }  
}
