package com.yiutil.tools;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.lang.ref.SoftReference;
import java.net.URL;
import java.security.MessageDigest;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.Environment;
import android.os.Handler;
import android.os.Message;
import android.os.StatFs;

/** @类名：CacheUtil
 *  @创建：
 *  @创建日期：
 *  @说明：
 ***************************************************/
@SuppressWarnings("serial")
public class CacheUtil{
	private static CacheUtil clientInstance;
	private static final String TAG="DownloadCache";
	private static final int FREE_SD_SPACE_NEEDED_TO_CACHE = 10*1024*1024;
	private static final int MB = 1024*1024;
	private static final int CACHE_SIZE = 10;
	private static final int HARD_CACHE_CAPACITY = 30;
	private static final int mTimeDiff = 400000;
    private FileUtil mFileHelper;
    private static String FILEPATH = "/sdcard/yiandroid/";
	/**引用次数，当次数为0时则释放自己*/
    private Context mAppContext;
    //缓存下载图片  
    private Map<String, String> cache = new HashMap<String, String>();//KEY:图片URL；VALUE:下载后的图片路径  
    //下载队列  
    private List<ImageDownloadItem> queue = new ArrayList<ImageDownloadItem>();  
    private final HashMap<String, Bitmap> mHardBitmapCache = 
    	new LinkedHashMap<String, Bitmap>(HARD_CACHE_CAPACITY/ 2, 0.75f, true){
		@Override
		protected boolean removeEldestEntry(LinkedHashMap.Entry<String, Bitmap> eldest) {
			boolean bret = false;
			if (size() >HARD_CACHE_CAPACITY){
				//当map的size大于30时，把最近不常用的key放到mSoftBitmapCache中，从而保证mHardBitmapCache的效率
				mSoftBitmapCache.put(eldest.getKey(), new SoftReference<Bitmap>(eldest.getValue()));
				bret = true;
			} 
			return bret;
		}
	};
	
    private synchronized void addDownloadItem(ImageDownloadItem item){  
        queue.add(item);  
        this.notify();//添加了下载项就激活本线程  
    } 
    
    public Bitmap downloadWithCache(ImageDownloadItem item) {  
        if (cache.containsKey(item.imageUrl)) {  
            Bitmap bitmap = BitmapFactory.decodeFile(cache.get(item.imageUrl));  
            return bitmap;  
        } else {  
            addDownloadItem(item);  
        }  
        return null;  
    } 
    
    public void downloadWithoutCache(ImageDownloadItem item) {  
        addDownloadItem(item);  
    }  
	
	/**
	*当mHardBitmapCache的key大于30的时候，会根据LRU算法把最近没有被使用的key放入到这个缓存中。
	*Bitmap使用了SoftReference，当内存空间不足时，此cache中的bitmap会被垃圾回收掉
	*/
	private final static ConcurrentHashMap<String, SoftReference<Bitmap>> mSoftBitmapCache = 
		new ConcurrentHashMap<String,SoftReference<Bitmap>>(HARD_CACHE_CAPACITY / 2); 
	/**
	 * 供外部调用，获取唯一实例
	 */
	public static CacheUtil getInstance(Context context){
		Context appContext = context.getApplicationContext();
		if(clientInstance==null ||clientInstance.mAppContext!=appContext){
			clientInstance = new CacheUtil(appContext);
		}
		return clientInstance;
	}
	
	public boolean isDownload(String imageUrl){  
	    return cache.containsKey(imageUrl);  
	}  
	
	private CacheUtil(Context context){
		mAppContext = context;
		mFileHelper = FileUtil.getInstance(mAppContext);
		//new Thread(this).start();
	}
	
	public void setCacheLocation(String strLoc){
		FILEPATH = strLoc;
	}
	
	public String convertUrlToFileName(String url){
		if(url==null ||url.equals("") ){
			return null;
		}
		MessageDigest md = null;
		try{
			md = MessageDigest.getInstance("MD5");		
			md.update(url.getBytes());
		}catch (Exception e){
			return url.substring(url.lastIndexOf('/') + 1, url.lastIndexOf('.'));
		}
		byte b[] = md.digest();
		int i;
		StringBuffer buf = new StringBuffer(64);
		for (int offset = 0; offset < b.length; offset++){
			i = b[offset] & 0xff;
			if (i < 16)
				buf.append('0');
			buf.append(Integer.toHexString(i));
		}
		return buf.toString();
	}
	
	private void saveImageToSd(Bitmap bm, String url,String dir){
		if(bm == null){
			LogUtil.LogShow(TAG, " trying to savenull bitmap",LogUtil.WARN);
			return;
		}
		
		//判断sd是否可存，判断sdcard上的空间是否足够
		if(mFileHelper.isCanSaveSdcard(FREE_SD_SPACE_NEEDED_TO_CACHE)==false){
			LogUtil.LogShow(TAG, "Low free space onsd, do not cache",LogUtil.WARN);
			return;
		}
		//removeCache(FILEPATH);
		mFileHelper.IsDirectory(dir);
		String filename = convertUrlToFileName(url);
		if(filename==null || filename.equals("")){
			return;
		}
		try{
			File file = new File(dir + filename);
			if(file.createNewFile()==false)
				return;
			OutputStream outStream = new FileOutputStream(file);
			//bm.compress(Bitmap.CompressFormat.JPEG, 100, outStream);
			bm.compress(Bitmap.CompressFormat.PNG, 100, outStream);
			
			outStream.flush();
			outStream.close();
			updateFileTime(dir,filename);
			LogUtil.LogShow(TAG, "Image saved tosd",LogUtil.WARN);
		} catch (FileNotFoundException e) {
			LogUtil.LogShow(TAG,"FileNotFoundException",LogUtil.WARN);
		} catch (IOException e){	
			LogUtil.LogShow(TAG,"IOException"+e,LogUtil.WARN);
		}
	} 
	
	/**
	* 计算sdcard上的剩余空间
	* @return
	*/
	private int freeSpaceOnSd(){
		StatFs stat = new StatFs(Environment.getExternalStorageDirectory() .getPath());
		double sdFreeMB = ((double)stat.getAvailableBlocks() * (double) stat.getBlockSize()) / MB;
		return (int) sdFreeMB;
	}
	
	/**
	* 修改文件的最后修改时间
	* @param dir
	* @param fileName
	*/
	private void updateFileTime(String dir,String fileName){
		if(mFileHelper.isSDaccess()){
			File file = new File(dir,fileName); 
			long newModifiedTime =System.currentTimeMillis();
			file.setLastModified(newModifiedTime);
		}
	}
	
	/**
	*计算存储目录下的文件大小，当文件总大小大于规定的CACHE_SIZE或者sdcard剩余空间小于FREE_SD_SPACE_NEEDED_TO_CACHE的规定
	* 那么删除40%最近没有被使用的文件
	* @param dirPath
	* @param filename
	*/
	public void removeCache(String dirPath){
		File dir = new File(dirPath);
		File[] files = dir.listFiles();
		if (files == null){
			return;
		}
		int dirSize = 0;
		for (int i = 0; i < files.length;i++){
			//if(files[i].getName().contains(WHOLESALE_CONV))
			{
				dirSize += files[i].length();
			}
		}
		if (dirSize > CACHE_SIZE * MB ||FREE_SD_SPACE_NEEDED_TO_CACHE > freeSpaceOnSd()){
			int removeFactor = (int) ((0.4 *files.length) + 1);
			Arrays.sort(files, new FileLastModifSort());
			LogUtil.LogShow(TAG, "Clear some expiredcache files ",LogUtil.WARN);
			for (int i = 0; i <removeFactor; i++){
				//if(files[i].getName().contains(WHOLESALE_CONV))
				{
					files[i].delete(); 
				}
			}
		}
	}
	/**
	* 删除过期文件
	* @param dirPath
	* @param filename
	*/
	public void removeExpiredCache(String dirPath, String filename){
		File file = new File(dirPath,filename);
		if (System.currentTimeMillis() -file.lastModified() > mTimeDiff){
			LogUtil.LogShow(TAG, "Clear some expiredcache files ",LogUtil.WARN);
			file.delete();
		}
	} 
	
	/**
	* 根据文件的最后修改时间进行排序 *
	*/
	class FileLastModifSort implements Comparator<File>{
		public int compare(File arg0, File arg1){
			if (arg0.lastModified() >arg1.lastModified()){
				return 1;
			} else if (arg0.lastModified() ==arg1.lastModified()){
				return 0;
			} else{
				return -1;
			}
		}
	}
	
	/**
	* 从缓存或SD卡中获取图片
	*/
	public Bitmap getBitmapFromCache(String url){
		if(url==null || url.equals(""))
			return null;

		Bitmap bitmap = getBitmapFromCacheNoSD(url);
		if(bitmap==null && mFileHelper.isSDaccess()){
			String filename = convertUrlToFileName(url);
			if(filename==null || filename.equals("")){
				return null;
			}
			//bitmap = getPicFromBytes(FileHelper.fileRead(FILEPATH + filename),false);
			try{
				bitmap = BitmapUtil.getBitmapByPath(
						FILEPATH + filename, BitmapUtil.getOptions(FILEPATH + filename), 
						MachineUtil.getDisplayMetrics(mAppContext)[0],
						MachineUtil.getDisplayMetrics(mAppContext)[1]);
				if(bitmap!=null){
					updateFileTime(FILEPATH,filename);
					synchronized (mHardBitmapCache){
						mHardBitmapCache.put(url,bitmap);
					}
				}
			} catch (FileNotFoundException e){
				e.printStackTrace();
			}  
		}
		return bitmap;
	}
	
	/**
	* 仅从缓存中获取图片
	*/
	public Bitmap getBitmapFromCacheNoSD(String url){
		// 先从mHardBitmapCache缓存中获取
		Bitmap bitmap=null;
		synchronized (mHardBitmapCache){
			bitmap = mHardBitmapCache.get(url);
			if (bitmap != null) {
				//如果找到的话，把元素移到linkedhashmap的最前面，从而保证在LRU算法中是最后被删除
				mHardBitmapCache.remove(url);
				mHardBitmapCache.put(url,bitmap);
				return bitmap;
			}
		}
		//如果mHardBitmapCache中找不到，到mSoftBitmapCache中找
		
		SoftReference<Bitmap> bitmapReference = mSoftBitmapCache.get(url);
		if (bitmapReference != null){
			bitmap = bitmapReference.get();
			if (bitmap != null){
				return bitmap;
			} else{
				mSoftBitmapCache.remove(url);
			}
		}
		return bitmap;
	}
	
	public Bitmap getPicFromBytes(byte[] bytes,boolean isThumbnail){
		Bitmap bitmap =null;
		if (bytes != null){
			BitmapFactory.Options opts = new BitmapFactory.Options(); 
			if(isThumbnail){
				opts.inSampleSize = 1;
			}
			try{
				bitmap = BitmapFactory.decodeByteArray(bytes, 0, bytes.length, opts);
			} catch(Exception e){
				e.printStackTrace();
			}
		}
		return bitmap;
	}
	
	public Bitmap loadDrawable(final String imageUrl,final ImageCallback callback,
			final boolean isThumbnail){
		final Handler handler=new Handler(){
			@Override
			public void handleMessage(Message msg){
				callback.imageLoaded((Bitmap) msg.obj, imageUrl);
			}
		};
		
		new Thread(){
			public void run(){
				//缓存读取失败，再从网络下载
				Bitmap bitmap = loadImageFromUrl(imageUrl);
				if(bitmap!=null){//表示SD卡不存在
					mHardBitmapCache.put(imageUrl,bitmap);
				}else{
					//SD卡存在，之文件中读取，同时更新文件修改时间，好排序
					String filename = convertUrlToFileName(imageUrl);
					bitmap = getPicFromBytes(mFileHelper.fileRead(FILEPATH +"/" + filename),isThumbnail);
					if(bitmap!=null)
					{
						updateFileTime(FILEPATH,filename);
						mHardBitmapCache.put(imageUrl,bitmap);
					}
				}
				handler.sendMessage(handler.obtainMessage(0,bitmap));	
			};
		}.start();
		return null;
	}
	
	/**
	 * 从网络中下载，如果SD卡能存，则存好，返回null，如不存在SD卡，刚将采样率设为8，返回bitmap
	 * @param imageUrl
	 * @return
	 */
	protected Bitmap loadImageFromUrl(String imageUrl){
		Bitmap bitmap = null;
		try{		
			 BitmapFactory.Options opts = new BitmapFactory.Options();
		     opts.inJustDecodeBounds = true;
		     bitmap = BitmapFactory.decodeStream(new URL(imageUrl).openStream(), null, opts);
		     int be = (int)(opts.outHeight / 200.0f);
		     if (be <= 0)
		         be = 1;
		     opts.inSampleSize = be;
		     opts.inJustDecodeBounds = false;
		     bitmap = BitmapFactory.decodeStream(new URL(imageUrl).openStream(), null, opts);
		     if (this.mFileHelper.isSDaccess()) {
		         LogUtil.LogShow("AppConnect", "mFileHelper.isSDaccess()", 113);
		         saveImageToSd(bitmap, imageUrl, FILEPATH);
		         bitmap.recycle();
		         bitmap = null;
		     }
		} catch (Exception e){
			e.printStackTrace();
		}
		return bitmap;
	}

	public interface ImageCallback{
		public void imageLoaded(Bitmap imageDrawable,String imageUrl);
	}

	public void run(){
		/* while(true)
		 {  
            while(queue.size() > 0)
            {  
                ImageDownloadItem item = queue.remove(0);  
                String imagePath = downloadImage(item.imageUrl);  
                //缓存图片路径  
                cache.put(item.imageUrl, imagePath);  
  
                if (item.downloadListener != null) 
                {
                	//需要执行回调来显示图片  
                    item.imagePath = imagePath;  
                    //交由UI线程处理  
                    Message msg = handler.obtainMessage();  
                    msg.obj = item;  
                    handler.sendMessage(msg);  
                }  
            }  
            try 
            {  
                synchronized(this)
                {  
                    this.wait();//没有下载项时等待  
                }  
            } 
            catch (InterruptedException e)
            {  
                e.printStackTrace();  
            }  
        }*/  
	}
	
	/*private Handler handler = new Handler()
	{  
        @Override  
        public void handleMessage(Message msg)
        {  
            ImageDownloadItem item = (ImageDownloadItem)msg.obj;  
            Bitmap bitmap = BitmapFactory.decodeFile(item.imagePath);  
            item.downloadListener.update(bitmap, item.imageUrl);  
        }  
    };
    
    private String downloadImage(String imageUrl)
    {  
        //不提供该方法代码  
        //下载部分应该有专门下载文件的类（如：FileDownloadUtil.download(imageUrl)）  
        return "";  
    } */ 
	
	public static class ImageDownloadItem {  
        public String imageUrl;//需要下载的图片URL  
        public String imagePath;//下载的后图片路径  
        //private PostDownloadListener downloadListener;//回调监听  
        public void setPostDownloadListener(PostDownloadListener listener){  
            //this.downloadListener = listener;  
        }  
	}
	
	public static interface PostDownloadListener{  
	    //策略模式，由子类实现  
	    public void update(Bitmap bitmap, String imageUrl);  
	}  
}
