package com.yiutil.tools;

import java.lang.reflect.InvocationTargetException;

import java.lang.reflect.Method;

import android.graphics.PixelFormat;
import android.hardware.Camera;
import android.hardware.Camera.PreviewCallback;
import android.os.Build;
import android.view.SurfaceHolder;

/** @类名：VolumeUtil
 *  @创建：
 *  @创建日期：
 *  @说明：
 ***************************************************/
public class CameraUtil{
	private static Camera instance;
	private static boolean useFrontFacingCamera;
	
	/**Default values*/
	private static int fps = 15;
	private static int width = 320;
	private static int height = 240;
	private static SurfaceHolder holder = null;
	private static PreviewCallback callback = null;
	
	private static final int MIN_SDKVERSION_addCallbackBuffer = 7;
	private static final int MIN_SDKVERSION_setPreviewCallbackWithBuffer = 7;
	private static final int MIN_SDKVERSION_setDisplayOrientation = 8;

	private static Method addCallbackBufferMethod = null;
	private static Method setDisplayOrientationMethod = null;
	protected static Method setPreviewCallbackWithBufferMethod = null;
	private static int sdkVersion=0;
	
	static{
		CameraUtil.useFrontFacingCamera = false;
	}
	
	static{
		if(getSDKVersion() >= CameraUtil.MIN_SDKVERSION_addCallbackBuffer){
			try{
				CameraUtil.addCallbackBufferMethod =
					             Camera.class.getMethod("addCallbackBuffer", byte[].class);
			} catch (Exception e){		
			} 
		}
		
		if(getSDKVersion() >= CameraUtil.MIN_SDKVERSION_setPreviewCallbackWithBuffer){
			try {
				CameraUtil.setPreviewCallbackWithBufferMethod = 
					  Camera.class.getMethod("setPreviewCallbackWithBuffer", PreviewCallback.class);
			} catch (Exception e){		
			} 
		}
				
		if(getSDKVersion() >= CameraUtil.MIN_SDKVERSION_setDisplayOrientation){
			try{
				CameraUtil.setDisplayOrientationMethod = 
					             Camera.class.getMethod("setDisplayOrientation", int.class);
			} catch (Exception e){
				
			} 
		}
	}
	
	public static Camera getCamera(){
		return CameraUtil.instance;
	}
	
	public static Camera openCamera(int fps, int width, int height, 
			                                 SurfaceHolder holder, PreviewCallback callback){
		if(CameraUtil.instance == null){
			try{
				if(CameraUtil.useFrontFacingCamera){
					CameraUtil.instance = CameraUtil.openFrontFacingCamera();
				}else{
					CameraUtil.instance = Camera.open();
				}
				
				CameraUtil.fps = fps;
				CameraUtil.width = width;
				CameraUtil.height = height;
				CameraUtil.holder = holder;
				CameraUtil.callback = callback;
				
				Camera.Parameters parameters = CameraUtil.instance.getParameters();

				parameters.setPreviewFormat(PixelFormat.YCbCr_420_SP);
				parameters.setPreviewFrameRate(CameraUtil.fps);
				CameraUtil.instance.setParameters(parameters);
				
				try{
					//parameters.setPictureSize(UMCameraProducer.width , UMCameraProducer.height);
					CameraUtil.instance.setParameters(parameters);
				}catch(Exception e){			
				}
				
				CameraUtil.instance.setPreviewDisplay(CameraUtil.holder);
				CameraUtil.initializeCallbacks(CameraUtil.callback);
			}catch(Exception e){
				CameraUtil.releaseCamera();	
			}
		}
		return CameraUtil.instance;
	}
	
	public static void releaseCamera(){
		if(CameraUtil.instance != null){
			CameraUtil.instance.setPreviewCallback(null);
			CameraUtil.instance.stopPreview();
			CameraUtil.deInitializeCallbacks();
			CameraUtil.instance.release();
			CameraUtil.instance = null;
		}
	}
	
	public static void setDisplayOrientation(int degrees){
		if(CameraUtil.instance != null && CameraUtil.setDisplayOrientationMethod != null){
			try{
				CameraUtil.setDisplayOrientationMethod.invoke(CameraUtil.instance, degrees);
			} catch (Exception e){
				
			}
		}
	}
	
	public static void setDisplayOrientation(Camera camera, int degrees){
		if(camera != null && CameraUtil.setDisplayOrientationMethod != null){
			try{
				CameraUtil.setDisplayOrientationMethod.invoke(camera, degrees);
			} catch (Exception e){
				
			}
		}
	}
	
	public static void addCallbackBuffer(Camera camera, byte[] buffer){
		try{
			CameraUtil.addCallbackBufferMethod.invoke(camera, buffer);
		} catch (Exception e){
			
		}
	}
	
	public static void addCallbackBuffer(byte[] buffer){
		try{
			CameraUtil.addCallbackBufferMethod.invoke(CameraUtil.instance, buffer);
		} catch (Exception e){
			
		}
	}

	public static boolean isAddCallbackBufferSupported(){
		return CameraUtil.addCallbackBufferMethod != null;
	}
	
	public static boolean isFrontFacingCameraEnabled(){
		return CameraUtil.useFrontFacingCamera;
	}
	
	public static void useRearCamera(){
		CameraUtil.useFrontFacingCamera = false;
	}
	
	public static void useFrontFacingCamera(){
		CameraUtil.useFrontFacingCamera = true;
	}
	
	public static Camera toggleCamera(){
		if(CameraUtil.instance != null){
			CameraUtil.useFrontFacingCamera = !CameraUtil.useFrontFacingCamera;
			CameraUtil.releaseCamera();
			
			CameraUtil.openCamera(CameraUtil.fps, 
					CameraUtil.width, 
					CameraUtil.height,
					CameraUtil.holder, 
					CameraUtil.callback);
		}
		return CameraUtil.instance;
	}
	
	private static void initializeCallbacks(PreviewCallback callback){
		if(CameraUtil.instance != null){
			/*if(UMCameraProducer.setPreviewCallbackWithBufferMethod != null)
			{
				try 
				{
					UMCameraProducer.setPreviewCallbackWithBufferMethod.invoke(UMCameraProducer.instance, callback);
				} 
				catch (Exception e) 
				{				
				}
			}
			else*/
			{
				CameraUtil.instance.setPreviewCallback(callback);
			}
		}
	}
	
	private static void deInitializeCallbacks(){
		if(CameraUtil.instance != null){
			/*if(UMCameraProducer.setPreviewCallbackWithBufferMethod != null)
			{
				try
				{
					UMCameraProducer.setPreviewCallbackWithBufferMethod.invoke(UMCameraProducer.instance, new Object[]{ null });
				} 
				catch (Exception e)
				{				
				}
			}
			else*/
			{
				CameraUtil.instance.setPreviewCallback(null);
			}
		}
	}
	
	public static Camera openFrontFacingCamera() 
	              throws IllegalArgumentException, IllegalAccessException, InvocationTargetException{
		Camera camera = null;	
		/**1. From mapper*/
		if((camera = FrontFacingCameraMapper.getPreferredCamera()) != null){
			return camera;
		}
		
		/**2. Use switcher*/
		if(FrontFacingCameraSwitcher.getSwitcher() != null){
			camera = Camera.open();
			FrontFacingCameraSwitcher.getSwitcher().invoke(camera, (int)1);
			return camera;
		}
		
		/** 3. Open front facing camera*/
		if((camera = FrontFacingCameraOpen.getFrontFacingCamera()) != null){
			return camera;
		}
		
		/**3. Use parameters*/
		camera = Camera.open();
		Camera.Parameters parameters = camera.getParameters();
		parameters.set("camera-id", 2);
		//parameters.setFocusMode("auto");
		//parameters.setSceneMode("");
		parameters.setFlashMode(Camera.Parameters.FLASH_MODE_ON);//Camera.Parameters.FLASH_MODE_TORCH);//�������
		//camera.autoFocus(Camera.Parameters.AutoFocusCallBack cb);
		camera.setParameters(parameters);
		return camera;
	}
	
	/*android 2.3 for  longqi just as Camera.open(flags);*/
	static class FrontFacingCameraOpen{
		static Camera getFrontFacingCamera(){
			try{
				Method method = Class.forName("android.hardware.Camera").getDeclaredMethod("open", int.class);
				return (Camera)method.invoke(null, 1);
			}catch(Exception e){
				
			}
			return null;
		}
	}
	
	/***
	 * FrontFacingCameraSwitcher
	 *
	 */
	static class FrontFacingCameraSwitcher{
		private static Method DualCameraSwitchMethod;
		
		static{
			try{
				FrontFacingCameraSwitcher.DualCameraSwitchMethod = Class.forName("android.hardware.Camera").getMethod("DualCameraSwitch",int.class);
			}catch(Exception e){
				
			}
		}
		
		static Method getSwitcher(){
			return FrontFacingCameraSwitcher.DualCameraSwitchMethod;
		}
	}
	
	/**
	 * FrontFacingCameraMapper
	 */
	static class FrontFacingCameraMapper{
		private static int preferredIndex = -1;
		
		static FrontFacingCameraMapper Map[] = {
			new FrontFacingCameraMapper("android.hardware.HtcFrontFacingCamera", "getCamera"),
			// Sprint: HTC EVO 4G and Samsung Epic 4G
			// DO not forget to change the manifest if you are using OS 1.6 and later
			new FrontFacingCameraMapper("com.sprint.hardware.twinCamDevice.FrontFacingCamera", "getFrontFacingCamera"),
			// Huawei U8230
            new FrontFacingCameraMapper("android.hardware.CameraSlave", "open"),
		};
		
		static{
			int index = 0;
			for(FrontFacingCameraMapper ffc: FrontFacingCameraMapper.Map){
				try{
					Class.forName(ffc.className).getDeclaredMethod(ffc.methodName);
					FrontFacingCameraMapper.preferredIndex = index;
					break;
				}catch(Exception e){
					
				}	
				++index;
			}
		}
		
		private final String className;
		private final String methodName;
		
		FrontFacingCameraMapper(String className, String methodName){
			this.className = className;
			this.methodName = methodName;
		}
		
		static Camera getPreferredCamera(){
			if(FrontFacingCameraMapper.preferredIndex == -1){
				return null;
			}
			
			try{
				Method method = Class.forName(FrontFacingCameraMapper.Map[FrontFacingCameraMapper.preferredIndex].className)
				.getDeclaredMethod(FrontFacingCameraMapper.Map[FrontFacingCameraMapper.preferredIndex].methodName);
				return (Camera)method.invoke(null);
			}catch(Exception e){
				
			}
			return null;
		}
	}
	
    public static int getSDKVersion(){
	   	if(sdkVersion == 0){
	   		sdkVersion = Integer.parseInt(Build.VERSION.SDK);
	   	}
	   	return sdkVersion;
    }
}

