package com.yiutil.tools;

import java.io.ByteArrayOutputStream;
import java.io.File;   
import java.io.FileInputStream;   
import java.io.FileOutputStream;   
import java.io.FileReader;
import java.io.IOException;   
import java.io.LineNumberReader;

import android.content.Context;   
import android.os.Environment;  
import android.os.StatFs;

/**
 * @类名：FileHelper
 * @创建：
 * @创建日期：
 * @说明：文件操作帮助类
 ***************************************************/
public final class FileUtil{ 
	private static FileUtil instance;
    private String sdcardpath;   
    private String filepath = "/data/data/com.mgyun.shua/files";
    private boolean issdacess;//sd卡是否可读
	private Context mAppContext;
	
    public synchronized static FileUtil getInstance(Context context){
        Context appContext = context.getApplicationContext();
        if(instance==null || instance.mAppContext!=appContext){
            instance = new FileUtil(appContext);
        }
        return instance;
    }
    
    private FileUtil(Context context) {   
    	mAppContext = context;
        issdacess = false;
        if (Environment.getExternalStorageState().equals(Environment.MEDIA_MOUNTED)){
        	 sdcardpath = Environment.getExternalStorageDirectory().getPath() + "/"; 
        	 issdacess = true;
        }       
    } 
    
    /**
     * 判断文件是否存在
     * @param strFileName
     */
    public boolean isFileExist(String strFileName){
    	boolean bret = false;
    	if(strFileName!=null && !strFileName.equals("")){
    		LogUtil.LogShow("FileHelper", "li strFileName="+strFileName, LogUtil.INFO);
        	File file = new File(strFileName);
        	LogUtil.LogShow("FileHelper", "li path="+file.getAbsolutePath(), LogUtil.INFO);
        	if(file.exists()){
        		LogUtil.LogShow("FileHelper", "li strFileName="+strFileName, LogUtil.INFO);
        		bret = true;
        	}
    	}
    	return bret;
    }
    
    /** 
	 * @param context
	 * @param strFilename
	 * @param data
	 */
    public void fileSave(Context context,String strFilename,byte[] data){
    	File saveFile = new File(strFilename);
    	FileOutputStream fileStream  = null;
        try {
              /**从得到文件流对象*/
        	 //if (Environment.getExternalStorageState().equals(Environment.MEDIA_MOUNTED))
        	 {
        		fileStream = new FileOutputStream(saveFile);
             	fileStream.write(data);
        	 }	
        }catch(Exception e) {      
            e.printStackTrace();     
        }
        finally{
           try{
        	   if(fileStream!=null) {
        		   fileStream.close(); 
        	   }  
           }catch (IOException e){
                e.printStackTrace();
           }
        } 
    }
    
    /**
     * 读取文件
     * @param context
     * @param fileName
     * @return
     */
    public byte[] fileRead(String fileName) {
    	LogUtil.LogShow("UMFileHelper", "fileRead1", LogUtil.INFO);
    	File saveFile = new File(fileName);
    	if(saveFile.exists()==false){
    		saveFile=null;
    		return null;
    	}
    	FileInputStream fis = null;
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
    	try{
    		if(Environment.getExternalStorageState().equals(Environment.MEDIA_MOUNTED)){
    			fis = new FileInputStream(saveFile);
    			byte[] buf = new byte[1024];
        		int len = 0;
        		/**将读取后的数据放置在内存中---ByteArrayOutputStream*/
        		while ((len = fis.read(buf)) != -1) {
        			baos.write(buf, 0, len);
        		}
        	}
    	}catch(Exception e){
    		  e.printStackTrace();
    	}finally{
    		try {
    			if(fis!=null){
    				fis.close();
    			}
    			baos.close();
            } catch (IOException e){
                 e.printStackTrace();
            }
    	}
    	LogUtil.LogShow("UMFileHelper", "fileRead2", LogUtil.INFO);
		return baos.toByteArray();
	} 
    
    /**
     * 
     * @param context
     * @param data
     */
    public void logToSDFile(Context context,String data){
    	File saveFile=new File("/sdcard/Test/Log.txt");
    	FileOutputStream fileStream   = null;
        try{
             /**从得到文件流对象*/
        	 if (Environment.getExternalStorageState().equals(Environment.MEDIA_MOUNTED)){
        		fileStream = new FileOutputStream(saveFile);   
             	fileStream.write(data.getBytes());//开始写入
        	 }      	
        }catch(Exception e) {      
            e.printStackTrace();     
        }finally {
           try {
        	   if(fileStream!=null){
        		   fileStream.close(); 
        	   }  
           } catch (IOException e) {
                e.printStackTrace();
           }
        } 
    }
  
    public boolean isSDaccess() {
    	return issdacess;
    	//return false;
    }
    
    /**
     * 判断是否可存在sd卡上
     * @param filesize
     * @return
     */
    public boolean isCanSaveSdcard(long filesize){
    	boolean bRet = false;
    	if(issdacess){
    		File path = Environment.getExternalStorageDirectory();
    		StatFs stat = new StatFs(path.getPath());
    		long blockSize = stat.getBlockSize(); 
            long availableBlocks = stat.getAvailableBlocks();
            if(filesize<availableBlocks*blockSize){
            	bRet = true;
            }
    	}
    	return bRet;
    }
    
    /**
     * 是否可写在手机内存中
     * @param filesize
     * @return
     */
    public boolean isCanSaveMemory(long filesize){
    	boolean bRet = false;
    	File path = Environment.getDataDirectory(); 
		StatFs stat = new StatFs(path.getPath());
		long blockSize = stat.getBlockSize(); 
		long availableBlocks = stat.getAvailableBlocks();
		if(filesize<availableBlocks*blockSize){
			bRet = true;
		}
    	return bRet;
    }
    
    /**  
     * 在SD卡上创建文件   
     */  
    public File creatFile(String fileName){   
    	File file =null;  
	    try{
	    	file = new File(fileName);
			file.createNewFile();
		} catch (IOException e){
	    	file = null;
			e.printStackTrace();
		} 
    	
        return file;   
    } 
  
    /**  
     * 在SD卡上创建文件   
     */  
    public File creatSDFile(String fileName) throws IOException{   
    	File file =null;  
    	if(issdacess){
    		file = new File(sdcardpath + fileName);
	        file.createNewFile(); 
    	}
        return file;   
    } 
    
    /**
     * 判断在sd卡目录是否存在，不存在则创建
     * @param dirName 是相对路径
     * @return
     */
    public File createSdDirectory(String dirName){
    	File dir = null;
    	try{
    		dir = new File(sdcardpath+dirName);
	 	    if(!dir.isDirectory()){
	 	    	dir.mkdirs();
	 	    }
 	    }catch(SecurityException e){
 	         e.printStackTrace();
 	    }
 	    LogUtil.LogShow("UMLogin", "create dir ="+dir.getAbsolutePath(), LogUtil.INFO);
    	return dir;
    }
    
    /**
     * 判断文件夹是否存在，不存在则创建
     */
    public boolean IsDirectory(String dirName){
    	try{
	 	    if(!(new File(dirName).isDirectory())){
	 	       new File(dirName).mkdirs();
	 	    }
 	    }catch(SecurityException e){
 	         e.printStackTrace();
 	    }
 	   return true;
    }
    
    /**  
     * 删除SD卡上的文件   
     */  
    public boolean IsSDFileExist(String fileName) {   
    	boolean bRet=true;
    	if(issdacess){
    		File file = new File(sdcardpath + fileName);   
 	        if (file == null || !file.exists() || file.isDirectory()){
 	        	bRet = false;   
 	        }
    	}else{
    		bRet = false;
    	}
        return bRet;   
    } 
    
    /**获取文件后缀名
     * @param fileName 文件名
     * @return 后缀 
     */ 
    public String getExt(String fileName){ 
    	int pos = fileName.lastIndexOf("."); 
    	if(pos==-1)
    		return ""; 
    	return fileName.substring(pos+1, fileName.length()); 
    } 
   
    /**
     * 返回sd卡
     * @return
     */
    public String getSDcardPath(){
    	return sdcardpath;
    }
    
    /**  
     * 删除SD卡上的文件   
     */  
    public boolean delSDFile(String fileName) {   
        File file = new File(sdcardpath + fileName);   
        if (file == null || !file.exists() || file.isDirectory()){
            return false;   
        }
        file.delete();   
        return true;   
    }   
  
    /**  
     * 在SD卡上创建目录  
     */  
    public File creatSDDir(String dirName) {   
        File dir = new File(sdcardpath + dirName); 
        if(!(dir.isDirectory())){
 	       dir.mkdirs();
 	    } 
        return dir;   
    }   
  
    /**  
     * 删除SD卡上的目录  
     */  
    public boolean delSDDir(String dirName) {   
        File dir = new File(sdcardpath + dirName);   
        return delDir(dir);   
    }   
  
    /**  
     * 修改SD卡上的文件或目录名  
     */  
    public boolean renameSDFile(String oldfileName, String newFileName) {   
        File oleFile = new File(sdcardpath + oldfileName);   
        File newFile = new File(sdcardpath + newFileName);   
        return oleFile.renameTo(newFile);   
    }   
  
    /**  
     * 拷贝SD卡上的单个文件  
     */  
    public boolean copySDFileTo(String srcFileName, String destFileName)   
            throws IOException {   
        File srcFile = new File(sdcardpath + srcFileName);   
        File destFile = new File(sdcardpath + destFileName);   
        return copyFileTo(srcFile, destFile);   
    }   
  
    /**  
     * 拷贝SD卡上指定目录的所有文件  
     */  
    public boolean copySDFilesTo(String srcDirName, String destDirName)   
            throws IOException         {   
        File srcDir = new File(sdcardpath + srcDirName);   
        File destDir = new File(sdcardpath + destDirName);   
        return copyFilesTo(srcDir, destDir);   
    }   
  
    /**  
     * 移动SD卡上的单个文件  
     */  
    public boolean moveSDFileTo(String srcFileName, String destFileName)   
            throws IOException {   
        File srcFile = new File(sdcardpath + srcFileName);   
        File destFile = new File(sdcardpath + destFileName);   
        return moveFileTo(srcFile, destFile);   
    }   
  
    /**  
     * 移动SD卡上的指定目录的所有文件  
     */  
    public boolean moveSDFilesTo(String srcDirName, String destDirName)   
            throws IOException {   
        File srcDir = new File(sdcardpath + srcDirName);   
        File destDir = new File(sdcardpath + destDirName);   
        return moveFilesTo(srcDir, destDir);   
    }   
  
  
    /*  
     * 将文件写入sd卡。如:writeSDFile("test.txt");  
       
    public Output writeSDFile(String fileName) throws IOException 
    {   
        File file = new File(SDPATH + fileName);   
        FileOutputStream fos = new FileOutputStream(file);   
        return new Output(fos);   
    } */  
  
    /*  
     * 在原有文件上继续写文件。如:appendSDFile("test.txt");   
    public Output appendSDFile(String fileName) throws IOException 
    {   
        File file = new File(SDPATH + fileName);   
        FileOutputStream fos = new FileOutputStream(file, true);   
        return new Output(fos);   
    } */   
  
    /*  
     * 从SD卡读取文件。如:readSDFile("test.txt");  
       
    public Input readSDFile(String fileName) throws IOException 
    {   
        File file = new File(SDPATH + fileName);   
        FileInputStream fis = new FileInputStream(file);   
        return new Input(fis);   
    }*/   
       
       
    /**  
     * 建立私有文件  
     */  
    public File creatDataFile(String fileName)  {   
    	File filePath = null;
    	try{
    		filePath = new File(fileName); 
    		filePath.getParentFile().mkdirs();
		    //file = new File(filepath +java.io.File.separator+fileName);  
    		filePath.createNewFile();
	        LogUtil.LogShow("UMLogin", "creatDataFile path1="+filePath.getAbsolutePath(), LogUtil.INFO);  
	        LogUtil.LogShow("UMLogin", "creatDataFile path2="+filePath.getAbsolutePath(), LogUtil.INFO); 
    	}catch(IOException e){
    		e.printStackTrace();
    	}catch(SecurityException e){
  	         e.printStackTrace();
  	    }
        
        return filePath;   
    } 
    
    /**  
     * 建立私有目录  
     */  
    public File createDataDirectory(String dirName) {   
    	File dir = null;
    	try{
    		dir = new File(filepath + dirName);
	 	    if(!dir.isDirectory()) {
	 	    	if(dir.isDirectory())
	 	    		LogUtil.LogShow("UMLogin", "data mkdir successfully", LogUtil.INFO);
	 	    	else
	 	    		LogUtil.LogShow("UMLogin", "data mkdir failed", LogUtil.INFO);
	 	    }
 	    }catch(SecurityException e) {
  	         e.printStackTrace();
  	    }
    	LogUtil.LogShow("UMLogin", "data directory="+dir.getAbsolutePath(), LogUtil.INFO);
    	return dir;   
    }   
  
    /**  
     * 删除私有文件  
     */  
    public boolean delDataFile(String fileName)  {   
        File file = new File(filepath + fileName);   
        return delFile(file);   
    }   
  
    /**  
     * 删除私有目录  
     */  
    public boolean delDataDir(String dirName) {   
        File file = new File(filepath + dirName);   
        return delDir(file);   
    }   
  
    /**  
     * 更改私有文件名  
     */  
    public boolean renameDataFile(String oldName, String newName) {   
        File oldFile = new File(filepath + oldName);   
        File newFile = new File(filepath + newName);   
        return oldFile.renameTo(newFile);   
    }   
  
    /**  
     * 在私有目录下进行文件复制  
     * @param srcFileName  
     *            ： 包含路径及文件名  
     */  
    public boolean copyDataFileTo(String srcFileName, String destFileName)   
            throws IOException  {   
        File srcFile = new File(filepath + srcFileName);   
        File destFile = new File(filepath + destFileName);   
        return copyFileTo(srcFile, destFile);   
    }   
  
    /**  
     * 复制私有目录里指定目录的所有文件  
     */  
   public boolean copyDataFilesTo(String srcDirName, String destDirName)   
            throws IOException {   
        File srcDir = new File(filepath + srcDirName);   
        File destDir = new File(filepath + destDirName);   
        return copyFilesTo(srcDir, destDir);   
    }   
  
    /**  
     * 移动私有目录下的单个文件  
     */  
    public boolean moveDataFileTo(String srcFileName, String destFileName)   
            throws IOException {   
        File srcFile = new File(filepath + srcFileName);   
        File destFile = new File(filepath + destFileName);   
        return moveFileTo(srcFile, destFile);   
    }   
  
    /**  
     * 移动私有目录下的指定目录下的所有文件  
     */  
    public boolean moveDataFilesTo(String srcDirName, String destDirName)   
            throws IOException{   
        File srcDir = new File(filepath + srcDirName);   
        File destDir = new File(filepath + destDirName);   
        return moveFilesTo(srcDir, destDir);   
    }   
    /**  
     * 删除一个文件  
     */  
    public boolean delFile(File file) {   
        if (file.isDirectory()) {
            return false;  
        }
        return file.delete();   
    }   
  
	/**
	 * 删除一个目录（可以是非空目录）
	 * 
	 * @param dir
	 */
	public boolean delDir(File dir){
		if (dir == null || !dir.exists() || dir.isFile()){
			return false;
		}else{
			for (File file : dir.listFiles()){
				if (file.isFile()){
					file.delete();
				}else if (file.isDirectory()){
					delDir(file);// 递归
				}
			}
			dir.delete();
			return true;
		}
	}
	
	/**
	 * 删除一个目录下的所有文件（可以是非空目录）
	 * 
	 * @param dir
	 */
	public boolean delDirAllFile(File fileDir){
		if (fileDir == null || !fileDir.exists() || fileDir.isFile())
		{
			return false;
		}else{
			for (File file : fileDir.listFiles()){
				if (file.isFile()){
					file.delete();
				}
			}
			return true;
		}
	}
	
	/**
	 * 拷贝一个文件,srcFile源文件，destFile目标文件
	 */
	public boolean copyFileTo(File srcFile, File destFile) throws IOException{
		if (srcFile.isDirectory() || destFile.isDirectory()){
			return false;// 判断是否是文件
		}else{
			try{
				FileInputStream fis = new FileInputStream(srcFile);
				FileOutputStream fos = new FileOutputStream(destFile);
				int readLen = 0;
				byte[] buf = new byte[1024];
				try{
					while ((readLen = fis.read(buf)) != -1){
						fos.write(buf, 0, readLen);
					}
					fos.flush();
				}finally{
					fis.close();
					fos.close();
				}
			}catch (Exception e){
				e.printStackTrace();
			}
			return true;
		}
	}
  
    /**  
     * 拷贝目录下的所有文件到指定目录  
     */  
    public boolean copyFilesTo(File srcDir, File destDir) throws IOException{   
        if (!srcDir.isDirectory() || !destDir.isDirectory())  {
            return false;// 判断是否是目录   
        }
        if (!destDir.exists())  
        {
            return false;// 判断目标目录是否存在   
        }
        File[] srcFiles = srcDir.listFiles();   
        for (int i = 0; i < srcFiles.length; i++){   
            if (srcFiles[i].isFile())  {   
                // 获得目标文件   
                File destFile = new File(destDir.getPath() + "\\"  
                        + srcFiles[i].getName());   
                copyFileTo(srcFiles[i], destFile);   
            }else if (srcFiles[i].isDirectory()){   
                File theDestDir = new File(destDir.getPath() + "\\"  
                        + srcFiles[i].getName());   
                copyFilesTo(srcFiles[i], theDestDir);   
            }   
        }   
        return true;   
    }   
  
    /**  
     * 移动一个文件  
     */  
    public boolean moveFileTo(File srcFile, File destFile) throws IOException {   
        boolean iscopy = copyFileTo(srcFile, destFile);   
        if (!iscopy)  {
            return false; 
        }
        delFile(srcFile);   
        return true;   
    }   
  
    /**  
     * 移动目录下的所有文件到指定目录 
     */  
    public boolean moveFilesTo(File srcDir, File destDir) throws IOException{   
        if (!srcDir.isDirectory() || !destDir.isDirectory()){   
            return false;   
        }   
        File[] srcDirFiles = srcDir.listFiles();   
        for (int i = 0; i < srcDirFiles.length; i++){   
            if (srcDirFiles[i].isFile()){   
                File oneDestFile = new File(destDir.getPath() + "\\"  
                        + srcDirFiles[i].getName());   
                moveFileTo(srcDirFiles[i], oneDestFile);   
                delFile(srcDirFiles[i]);   
            }else if (srcDirFiles[i].isDirectory()){   
                File oneDestFile = new File(destDir.getPath() + "\\"  
                        + srcDirFiles[i].getName());   
                moveFilesTo(srcDirFiles[i], oneDestFile);   
                delDir(srcDirFiles[i]);   
            }   
        }   
        return true;   
    } 
    
    // 文件内容的总行数。 
    static int getTotalLines(File file) throws IOException{
        FileReader in = new FileReader(file);
        LineNumberReader reader = new LineNumberReader(in);
        String s = reader.readLine();
        int lines = 0;
        while (s != null){
            lines++;
            s = reader.readLine();
        }
        reader.close();
        in.close();
        return lines;
    }
    
    // 读取文件指定行。 
    static void readAppointedLineNumber(File sourceFile, int lineNumber) throws IOException{
        FileReader in = new FileReader(sourceFile);
        LineNumberReader reader = new LineNumberReader(in);
        String s = reader.readLine();
        
        if (lineNumber < 0 || lineNumber > getTotalLines(sourceFile)){
            System.out.println("不在文件的行数范围之内。");
        }
        {
	        while (s != null){
	            System.out.println("当前行号为:" + reader.getLineNumber());
	            
	            System.out.println(s);
	            System.exit(0);
	            s = reader.readLine();
	        }
        }
        reader.close();
        in.close();
    }
}

