package com.yiutil.tools;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.net.InetAddress;
import java.net.NetworkInterface;
import java.net.SocketException;
import java.util.Enumeration;

import com.yiutil.bean.YiUtil;

import android.content.Context;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.content.pm.PackageManager.NameNotFoundException;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.net.wifi.WifiInfo;
import android.net.wifi.WifiManager;
import android.os.Environment;
import android.os.StatFs;
import android.telephony.TelephonyManager;
import android.text.TextUtils;
import android.util.DisplayMetrics;
import android.util.Log;
import android.view.WindowManager;

/** @类名：MachineUtil
 *  @创建：
 *  @创建日期：
 *  @说明：
 ***************************************************/
public class MachineUtil{
	private static MachineUtil clientInstance=null;
	/**
	 * 供外部调用，获取唯一实例
	 */
	public static synchronized MachineUtil getInstance(){
		if(clientInstance==null){
			clientInstance = new MachineUtil();
		}
		return clientInstance;
	}
	
	private MachineUtil(){
		
	}
	
	public static String getLocalMacAddress(Context context){   
        WifiManager wifi = (WifiManager) context.getSystemService(Context.WIFI_SERVICE);   
        WifiInfo info = wifi.getConnectionInfo();   
        return info.getMacAddress();   
    } 
	
	/**
	 * 获取cpu信息
	 * @return
	 */
	public static String[] getCpuInfo() {  
        String str1 = "/proc/cpuinfo";  
        String str2="";  
        String[] cpuInfo={"",""};  
        String[] arrayOfString;  
        try{  
            FileReader fr = new FileReader(str1);  
            BufferedReader localBufferedReader = new BufferedReader(fr, 8192);  
            str2 = localBufferedReader.readLine();  
            arrayOfString = str2.split("\\s+");  
            for (int i = 2; i < arrayOfString.length; i++){  
                cpuInfo[0] = cpuInfo[0] + arrayOfString[i] + " ";  
            }  
            str2 = localBufferedReader.readLine();  
            arrayOfString = str2.split("\\s+");  
            cpuInfo[1] += arrayOfString[2];  
            localBufferedReader.close();  
        } catch (IOException e) {  
        }  
        return cpuInfo;  
    }
	
	/**
	 * 获取版本号
	 * @param context
	 * @param _packageName
	 * @return
	 */
	public static int getVerCode(Context context,String _packageName){
        int verCode = -1;
        try{
            verCode = context.getPackageManager().getPackageInfo(
            		_packageName, 0).versionCode;
        } catch (NameNotFoundException e) {
            
        }
        return verCode;
    }

	/**
	 * 获取版本名称
	 * @param _context
	 * @param _packageName
	 * @return
	 */
	public static String getVerName(Context _context,String _packageName){  
        PackageManager packageManager = _context.getPackageManager();  
        String name = null;  
        try {  
            name = packageManager.getPackageInfo(_packageName, 0).versionName;  
        } catch (PackageManager.NameNotFoundException e){  
            e.printStackTrace();  
        }  
        return name;  
    } 
	
	/**
	 * 获取使用的网络类型
	 * @param _context
	 * @return
	 */
	public static String getNetWorkType(Context _context){
	    ConnectivityManager conManager = (ConnectivityManager)_context.getSystemService("connectivity");
	    NetworkInfo netInfo = conManager.getActiveNetworkInfo();
	    if (netInfo == null)
	      return "unknow";
	    if (netInfo.getType() == 1)
	      return "wifi";
	    if (netInfo.getType() == 0)
	      return "2G/3G";
	    return "unknow";
	}
	
	/**
	 * 
	 * @param _context
	 * @return
	 */
	public static int getNetWorkTypeEx(Context _context){
	    int netType = 0;
	    ConnectivityManager conManager = (ConnectivityManager)_context.getSystemService("connectivity");
	    NetworkInfo netInfo = conManager.getActiveNetworkInfo();
	    if (netInfo != null) {
	      if (netInfo.getType() == 1)
	        netType = 1;
	      else if (netInfo.getType() == 0) {
	        netType = 2;
	      }
	    }
	    return netType;
	}
	
	/**
	 * 获取运营商名称
	 * @param _context
	 * @return
	 */
	public static String getSimOperator(Context _context){
	    TelephonyManager teleManager = (TelephonyManager)_context.getSystemService("phone");
	    String str = teleManager.getSubscriberId();
	    if ((str == null) || (TextUtils.isEmpty(str)))
	    	str = teleManager.getSimOperator();
	    if (str != null){
	    	if ((str.startsWith("46000")) || (str.startsWith("46002")))
	    		return "中国移动";
	    	if (str.startsWith("46001"))
	    		return "中国联通";
	    	if (str.startsWith("46003"))
	    		return "中国电信";
	    }
	    return "unknow";
	}
	
	/**
	 * 获取手机imsi
	 * @param paramContext
	 * @return
	 */
	public static String getSubscriberId(Context paramContext){
	    TelephonyManager teleManager = (TelephonyManager)paramContext.getSystemService("phone");
	    return teleManager.getSubscriberId();
	}
	
	/**
	 * 获取屏幕宽高
	 * @param _context
	 * @return
	 */
	public static int[] getDisplayMetrics(Context _context){
	    int[] windowMetrics = new int[2];
	    DisplayMetrics displayMetrics = new DisplayMetrics();
	    WindowManager localWindowManager = (WindowManager)_context.getSystemService("window");
	    localWindowManager.getDefaultDisplay().getMetrics(displayMetrics);
	    int i = displayMetrics.widthPixels;
	    int j = displayMetrics.heightPixels;
	    windowMetrics[0] = i;
	    windowMetrics[1] = j;
	    return windowMetrics;
	}
	
	//获取ip地址
	@SuppressWarnings("rawtypes")
	public static String getIpAddress(){
	    try{
			Enumeration enumInterface = NetworkInterface.getNetworkInterfaces();
	    	while (enumInterface.hasMoreElements()){
	    		NetworkInterface localNetworkInterface = (NetworkInterface)enumInterface.nextElement();
	    		Enumeration enumInet = localNetworkInterface.getInetAddresses();
	    		while (enumInet.hasMoreElements()){
	    			InetAddress inetItem = (InetAddress)enumInet.nextElement();
	    			if (!inetItem.isLoopbackAddress())
	    				return inetItem.getHostAddress().toString();
	    		}
	    	}
	    }catch (SocketException exception){
	      
	    }
	    return null;
	}
	
	//获取未安装软件的versionCode
	public static int getVersionCode(Context _context, String _packageName){
	    try {
	    	PackageInfo packageInfo = _context.getPackageManager().getPackageInfo(
	    			_packageName,  PackageManager.GET_UNINSTALLED_PACKAGES);
	    	if (packageInfo != null)
	    		return packageInfo.versionCode;
	    }catch (PackageManager.NameNotFoundException e){
	    	e.printStackTrace();
	    }
	    return -1;
	}
	
	//获取未安装应用的名字
	public static String getAppName(Context _context){
	    try {
	    	String str = _context.getPackageName();
	    	PackageManager packageManager = _context.getPackageManager();
	    	PackageInfo packageInfo = packageManager.getPackageInfo(str,
	    			PackageManager.GET_UNINSTALLED_PACKAGES);
	    	if ((packageInfo != null) && (packageInfo.applicationInfo != null)){
	    		CharSequence csLabel = packageInfo.applicationInfo.loadLabel(packageManager);
	    		StringBuffer s = new StringBuffer();
	    		s.append(csLabel);
	    		return s.toString();
	    	}
	    } catch (PackageManager.NameNotFoundException e){
	    	e.printStackTrace();
	    }
	    return null;
	}
	
	//获取sd卡路径
	public static String getSDPath(){
	    File sdDir = null;
	    boolean sdCardExist = Environment.getExternalStorageState().equals("mounted");
	    if (sdCardExist) {
	       sdDir = Environment.getExternalStorageDirectory();
	    }
	    return sdDir != null ? sdDir.toString() : null;
	}
	
	//判断sd卡是否挂载或者够空间
	public static int isSdPresent(){
		if (Environment.getExternalStorageState().equals(Environment.MEDIA_MOUNTED)) {
	        StatFs stat = new StatFs(Environment.getExternalStorageDirectory().getPath());
	        long blockSize = stat.getBlockSize();     
	        long availableBlocks = stat.getAvailableBlocks(); 
	        float megAvailable = blockSize*availableBlocks/ 1048576L;
	        if (megAvailable > 1) {
	           return YiUtil.SDCARD_STATE_MOUNTED;
	        }
	        return YiUtil.SDCARD_STATE_UNMOUNTED;
	    }
	    return YiUtil.SDCARD_STATE_UNMOUNTED;
	}
	
	//获取手机sim卡的状态
	@SuppressWarnings("static-access")
	public static boolean isSimOk(Context context){
    	boolean bret = false;
    	TelephonyManager telMgr = (TelephonyManager)context.getSystemService(
    			           Context.TELEPHONY_SERVICE); 
    	if(telMgr.getSimState()==telMgr.SIM_STATE_READY) {//ok 
    		bret = true;
    		Log.i("AppConnect", "sim ok");
    	} else if(telMgr.getSimState()==telMgr.SIM_STATE_ABSENT) {//无SIM卡; 
    		Log.i("AppConnect", "无SIM卡");
    	} else { //SIM卡被锁定或未知的状态;
    		Log.i("AppConnect", "SIM卡被锁定或未知的状态");
    	} 

    	return bret;
    }
	
	//获取手机号码
	public static String getPhoneNumber(Context context){
		String phoneNumber = null;
		TelephonyManager telMgr = (TelephonyManager)context.getSystemService(
		           Context.TELEPHONY_SERVICE); 
		phoneNumber = telMgr.getLine1Number();
		return phoneNumber;
	}
	
	//获取imei
	public static String getDeviceId(Context context){
		String phoneNumber = null;
		TelephonyManager telMgr = (TelephonyManager)context.getSystemService(
		           Context.TELEPHONY_SERVICE); 
		phoneNumber = telMgr.getDeviceId();
		return phoneNumber;
	}
	
	//获取国家码
	public static String getSimCountryIso(Context context){
		String phoneNumber = null;
		TelephonyManager telMgr = (TelephonyManager)context.getSystemService(
		           Context.TELEPHONY_SERVICE); 
		phoneNumber = telMgr.getSimCountryIso();
		return phoneNumber;
	}
	
	//获取version code
	public static int getVersionCode(Context _context){
	    try{
	    	PackageInfo info = _context.getPackageManager().getPackageInfo(
	    			_context.getPackageName(), 0);
	    	return info.versionCode;
	    }catch (PackageManager.NameNotFoundException e) {
	    }
	    return 0;
	}
	
	//获取VersionName
	public static String getVersionName(Context _context){
	    try{
	    	PackageInfo info = _context.getPackageManager().getPackageInfo(
	    			_context.getPackageName(), 0);
	    	return info.versionName;
	    }catch (PackageManager.NameNotFoundException e){
	    }
	    return null;
	}
	
	// 获取Country
	public static String getCountry(Context _context){
	    String str = _context.getResources().getConfiguration().locale.getCountry();
	    return str;
	}
	
	/**
	 * 获取网络类型
	 * @param _ctx
	 * @return
	 */
	public static int getNetType(Context _ctx){
		int iNetType = 0;
		ConnectivityManager connectivity = 
	    (ConnectivityManager) _ctx.getSystemService(Context.CONNECTIVITY_SERVICE);
		if (connectivity == null){
	        Log.e("MachineUtil", "获取网络类型失败");
	        iNetType = 0;
	    } else{
	        NetworkInfo info = connectivity.getActiveNetworkInfo();
	        if(info == null){
	        	Log.e("MachineUtil", "获取网络类型失败");
	        	iNetType = 0;
	        }else{
	        	if (info.getType() == ConnectivityManager.TYPE_MOBILE){
	 	        	TelephonyManager tm = (TelephonyManager)
	 	        			_ctx.getSystemService(Context.TELEPHONY_SERVICE);
	 	            int type = tm.getNetworkType();
	 	        	switch (type){
	 					case TelephonyManager.NETWORK_TYPE_EDGE:{
	 						iNetType = 2;
	 						Log.d("MachineUtil", "网络类型为EDGE");
	 					}break;
	 					case TelephonyManager.NETWORK_TYPE_GPRS:{
	 						iNetType = 1;
	 						Log.d("MachineUtil", "网络类型为GPRS");
	 					}break;
	 					case TelephonyManager.NETWORK_TYPE_UMTS:{
	 						iNetType = 2;
	 						Log.d("MachineUtil", "网络类型为UMTS");
	 					}break;
	 					case TelephonyManager.NETWORK_TYPE_UNKNOWN:{
	 						iNetType = 1;
	 						Log.d("MachineUtil", "网络类型未知");
	 					}break;
	 				}
	 	        } else if (info.getType() == ConnectivityManager.TYPE_WIFI){
	 	        	iNetType = 2;
	 	        	Log.d("MachineUtil", "网络类型为WIFI");
	 	        }
	        }   
	    }
		
		return iNetType;
	}
}
